#define ana2011_cxx
// The class definition in ana2011.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called every time a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("ana2011.C")
// Root > T->Process("ana2011.C","some options")
// Root > T->Process("ana2011.C+")
//

#include "ana2011.h"
#include <TH2.h>
#include <TStyle.h>

//ADD

#include <iostream>
#include <fstream>
#include <math.h>
#include <utility>

using namespace std;

#include "TFitter.h"
#include "TMath.h"
#include <TVector3.h>
#include <TVector2.h>

#include "JetID_v2.0.hpp"
#include "MCttCategories.hpp"
#include "EventShape.h"
#include "Thrust_func_collection.h"
#include "CaloIsoCorrection.cxx"
#include "muon_SF_LP11.h"

#include "PileupReweighting/PileupReweighting/TPileupReweighting.h"

#define PRINT 0
#define m_debug false
#define mjet0 false

#define ksigma 1
#define sigmaW 2000
#define sigmadmt 4000

#define RealData false
#define mcWjets true
#define doMatch true

//Functions for class ana2011
#include "chi2calculation.C"
#include "Functions.C"
#include "MatchTrigger.C"
#include "ana2011AddFunctions.C"
#include "FunctionsFlavourChi2.C"
#include "emuOverlap.C"
#include "SF.C"
#include "AddMuonJetFunction.C"

bool sortdecreaseSV0(TVector2 v1, TVector2 v2)
{
  if (v1.Y()>=v2.Y()) return true;
  else return false;
}

bool sortdecrease(Particle p1, Particle p2)
{
  TLorentzVector v1 = p1.vector4();
  TLorentzVector v2 = p2.vector4();  
  if (v1.Pt()>=v2.Pt()) return true;
  else return false;
}

bool sortincreaseDouble(double a, double b)
{
  if (a<b) return true;
  else return false;
}

void ana2011::Begin(TTree * /*tree*/)
{
  // The Begin() function is called at the start of the query.
  // When running with PROOF Begin() is only called on the client.
  // The tree argument is deprecated (on PROOF 0 is passed).

  TString option = GetOption();
  //make table of values for pTtt, probability, chi2
  //maketable();

  if (RealData){
    cout << "**********************************" << endl;
    cout << "********    RealData !!!  ********" << endl;
    cout << "**********************************" << endl;
  }
  if (mcWjets){
    cout << "**********************************" << endl;
    cout << "********    mcWjets !!!   ********" << endl;
    cout << "**********************************" << endl;
  }  

}

void ana2011::SlaveBegin(TTree * /*tree*/)
{
  // The SlaveBegin() function is called after the Begin() function.
  // When running with PROOF SlaveBegin() is called on each slave server.
  // The tree argument is deprecated (on PROOF 0 is passed).

  TString option = GetOption();

  // Initialise the pile-up reweighting tool
  m_PUrewei = new Root::TPileupReweighting( "ThePileUpRWTool" );
  int isGood = m_PUrewei->initialize( "ilumicalc_histograms_None_178044-184169.root",
				       "avgintperbx",
				       "mymu_mc10b.root",
				       "mymu_mc10b" );
  if (isGood != 0) std::cout << "Cannot initialize PileUpReweighting tool properly. Stop and check your inputs" << std::endl;

  //Initialize the PDF set and PDF number of members
  //iDefaultPdf = 20650, nMemDefaultPdf = 1;
  iTestPdf = 20650, nMemTestPdf = 1;
  ifstream ftxt("testPdf.txt");
  ftxt >> iTestPdf >> nMemTestPdf;
  ftxt.close();
  //Add PDF
  //m_checkPDF = new PDFReweight(20650,1,iTestPdf,nMemTestPdf);
  //LHAPDF::initPDFSet(3,20650,0);  


  //##############  TOOLS & SF & SMEARING ##############
  m_eleSF = new egammaSFclass();
  m_eleEscalesmear = new EnergyRescaler();
  m_eleEscalesmear->useDefaultCalibConstants("2011");
  m_muPsmear = new SmearingClass("muid","MuonMomentumCorrections/root/data/");
  m_muSF = new Analysis::MuidCBScaleEffFactors();
  m_btagSF = new Analysis::CalibrationDataInterfaceROOT("SV0", "CalibrationDataInterface/scripts/BTagCalibration.env"); 
  m_aJetForSF.jetAuthor = "AntiKt4Topo";

  m_ptElSm = 0.;
  m_ptMuSm = 0.;


  //################### FOR HISTOS ######################
  #include "InitHistos.h"

  //looses 
  m_loose = new TH1F("loose", "loose", 202, -2, 200);
  fOutput->Add(m_loose);
  
  m_looseE = new TH1F("looseE", "looseE", 202, -2, 200);
  fOutput->Add(m_looseE);
  
  m_looseM = new TH1F("looseM", "looseM", 202, -2, 200);
  fOutput->Add(m_looseM);
    
  m_looseSig = new TH1F("looseSig",   "", 202, -2, 200);
  fOutput->Add(m_looseSig);
  
  m_looseESig = new TH1F("looseESig", "", 202, -2, 200);
  fOutput->Add(m_looseESig);
  
  m_looseMSig = new TH1F("looseMSig", "", 202, -2, 200);
  fOutput->Add(m_looseMSig);
    
  //jet multiplicity
  m_njets = new TH1F("njets",       "njets", 200, 1, 201);
  fOutput->Add(m_njets);
  
  m_njetsE = new TH1F("njetsE",    "njetsE", 200, 1, 201);
  fOutput->Add(m_njetsE);
  
  m_njetsM = new TH1F("njetsM",    "njetsM", 200, 1, 201);
  fOutput->Add(m_njetsM);

  m_njetsB = new TH1F("njetsB",    "njetsB", 200, 1, 201);
  fOutput->Add(m_njetsB);
  
  m_njetsBE = new TH1F("njetsBE", "njetsBE", 200, 1, 201);
  fOutput->Add(m_njetsBE);
  
  m_njetsBM = new TH1F("njetsBM", "njetsBM", 200, 1, 201);
  fOutput->Add(m_njetsBM);

  //loose for muon
  m_looseMuon = new TH1F("looseMuon", "looseMuon", 20, 0, 20);
  fOutput->Add(m_looseMuon);
  //loose for electron
  m_looseEle  = new TH1F("looseEle", "looseEle", 20, 0, 20);
  fOutput->Add(m_looseEle);

  //all that concern the weight
  m_lbn = new TH1F("lbn", "", 20, 0, 20);
  fOutput->Add(m_lbn);

  m_PUwei = new TH1F("PUwei", "", 50, 0, 5);
  fOutput->Add(m_PUwei);

  m_lbnW = new TH1F("lbnW", "", 20, 0, 20);
  fOutput->Add(m_lbnW);

  hwSFb = new TH1F("wSFb",  "", 20, 0, 2);
  fOutput->Add(hwSFb);

  hwSFemu  = new TH1F("wSFemu",   "", 20, 0, 2);
  fOutput->Add(hwSFemu);


  //#############  FILE AND TREE_ANA  #############
  file     = new TFile("res.root","RECREATE");
  tree_ana = new TTree("T","T");
  #include "SetBranchTreeAna.h"

  //***** Counters *****
  m_NAllFit = 0;
  m_nall = 0;
  n6DifPartons = 0;
  iEvtSel = 0;
  nHyperNasty = 0;
  nHasJetInHole = 0;

  //Bool for tools
  m_ApplyHoleVeto = true;
  UseTool1        = true; //mcevt
  UseTool2        = true; //Pileup reweighting
  UseTool3        = true; //Eff SF el,mu
  UseTool4        = true; //Eff SF btag
  m_smearMC       = true; //smear E of el and pt of mu
  m_corrData      = true; //for data only

  m_trigMu        = true;
  if (!RealData) 
    m_trigMu = false;
 
  EvtListC2  = fopen("Cut2.txt", "w+");
  EvtListC3  = fopen("Cut3.txt", "w+");
  EvtListC4  = fopen("Cut4.txt", "w+");
  EvtListC5  = fopen("Cut5.txt", "w+");
  EvtListC11 = fopen("Cut11.txt","w+");
  EvtListC12 = fopen("Cut12.txt","w+");
  EvtListC13 = fopen("Cut13.txt","w+");
  EvtListC14e.open("Cut14e.txt");
  EvtListC14m.open("Cut14m.txt");

  testptSEl1 = fopen("testptSEl1.txt","w+");
  testptSEl2 = fopen("testptSEl2.txt","w+");
  testptSMu1 = fopen("testptSMu1.txt","w+");
  testptSMu2 = fopen("testptSMu2.txt","w+");
} 

Bool_t ana2011::Process(Long64_t entry)
{
  // The Process() function is called for each entry in the tree (or possibly
  // keyed object in the case of PROOF) to be processed. The entry argument
  // specifies which entry in the currently loaded tree is to be processed.
  // It can be passed to either ana2011::GetEntry() or TBranch::GetEntry()
  // to read either all or the required parts of the data. When processing
  // keyed objects with PROOF, the object is already loaded and is available
  // via the fObject pointer.
  //
  // This function should contain the "body" of the analysis. It can contain
  // simple or elaborate selection criteria, run algorithms on the data
  // of the event and typically fill histograms.
  //
  // The processing can be stopped by calling Abort().
  //
  // Use fStatus to set the return value of TTree::Process().
  //
  // The return value is currently not used.

  //fChain->GetTree()->GetEntry(entry);
  //gROOT->SetStyle("Plain");
  //InitMyTree();

  double weight = 1;
  double weightEvent = 1;

  //MATCH :: Match MCParticles
  unsigned int ncate = -1;
  int Ite = -1;
  vector<TLorentzVector> MatchedPars;
  vector<Particle> mcpars;
  vector<Particle> resParsMC;

  if ((!RealData)&&(doMatch)) {
    GetSelBranch(entry,6);  
    for (int i = 0; i < mc_n; i++){
      TLorentzVector vp;
      if (mc_pt->at(i)==0) mc_pt->at(i) = 1e-5;
      vp.SetPtEtaPhiM(mc_pt->at(i),mc_eta->at(i),mc_phi->at(i),mc_m->at(i));

      int pdg = mc_pdgId->at(i);
      int stat = mc_status->at(i);
      int pol = mc_barcode->at(i);

      Particle p(vp,pdg,stat,pol);

      std::vector<int> dau = mc_child_index->at(i);
      p.child_index(dau);
      mcpars.push_back(p);
    }
    //To take out 12 fermions
    std::pair<unsigned int, std::vector<Particle> > mcCat = MCttCategories::ttCategories(mcpars);
    ncate = mcCat.first;
    vector<Particle> resPars = mcCat.second;
    
    for (unsigned int i = 0; i < resPars.size(); i++){
      Particle p = resPars.at(i);
      resParsMC.push_back(p);
    }

    m_category[0]->Fill(ncate,weight);
    b_catAll = ncate;

    //To take out the reconstructed objects
    std::pair<unsigned int, vector<TLorentzVector> > IteMatchedPars = matchQstat2Jets(resPars,mcpars,true);
    
    Ite = IteMatchedPars.first;
    MatchedPars = IteMatchedPars.second;
    if (Ite >= 100)
      m_NotMatch[0]->Fill(Ite/100);
    else 
      m_MergedJet[0]->Fill(Ite);
  }//End of initializing matching informations


  //Especially for W+jets and Wbb and Wcc and Wc
  b_top_hfor_type->GetEntry(entry);
  b_hfor = top_hfor_type; 
  m_hfor[0]->Fill(top_hfor_type);

  b_ApplyHoleVeto = (int)m_ApplyHoleVeto; 
  b_UseTool1      = (int)UseTool1;      
  b_UseTool2      = (int)UseTool2;      
  b_UseTool3      = (int)UseTool3;      
  b_UseTool4      = (int)UseTool4;      
  b_smearMC       = (int)m_smearMC;       
  b_corrData      = (int)m_corrData;      

  bool isEleEvent = false;

  GetRunNumberBranch(entry);
  GetSelBranch(entry,0);

  //****** ATTENTION AU LArHole ******

  double L1 = 0.852;
  gRandom->SetSeed(EventNumber);
  double rL1 = gRandom->Uniform();
  hasLArHole = false;
  if (rL1 < L1) hasLArHole = true;  
  b_hasLArHole = hasLArHole ? 1 : 0;

  //**********************************

  m_loose->Fill(-1);
  m_looseE->Fill(-1);
  m_looseM->Fill(-1);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(-1);
    if (ncate==12||ncate==21)
      m_looseESig->Fill(-1);
    if (ncate==13||ncate==31)
      m_looseMSig->Fill(-1);
  }

  //MC weight
  if (!RealData) { 
    b_mcevt_weight->GetEntry(entry);
    if (mcevt_weight->size()>=1 && (mcevt_weight->at(0)).size()>=1)
      weightEvent = (mcevt_weight->at(0)).at(0);
  }
  if (UseTool1)
    weight *= weightEvent;
  b_mcevt = weightEvent;         

  //PileUp Reweighting
  float PUwei = 1.;
  if (!RealData){
    if (lbn==100) lbn = 0;
    PUwei = m_PUrewei->getPileupWeight(lbn);
    //TEST PUweight
    if (UseTool2)
      weight *= PUwei;
    
    m_lbn->Fill(lbn);
    m_PUwei->Fill(PUwei);
    m_lbnW->Fill(lbn,PUwei);
    b_lbn_ana  = lbn;
    
  }

  m_loose->Fill(0.,weight);
  m_looseE->Fill(0.,weight);
  m_looseM->Fill(0.,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(0.,weight);
    if (ncate==12||ncate==21)
      m_looseESig->Fill(0.,weight);
    if (ncate==13||ncate==31)
      m_looseMSig->Fill(0.,weight);
  }

  b_PUwei = PUwei;         

  if (entry%10000 == 0) std::cout << "Reading event " << entry << " " << EventNumber << std::endl;
  //cout << fChain->GetTree()->GetCurrentFile()->GetName() << endl;

  if (RealData && RunNumber <= 160879) return kTRUE;

  //#####################################################################
  //######################    BEGIN PRESELECTION   ######################
  //#####################################################################  

  b_el_truth_barcode->GetEntry(entry);

  //C0 : Trigger
  bool TriggerMu = (EF_mu18||(!m_trigMu));

  //Reproduce CutFlow 3 August
  //m_trigMu = true;
  //TriggerMu = EF_mu18;

  if (!(EF_e20_medium || TriggerMu)) return kTRUE;
  //below is the check the cut flow
  m_loose->Fill(1,weight);
  if (EF_e20_medium)
    m_looseE->Fill(1,weight);
  if (TriggerMu)
    m_looseM->Fill(1,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(1,weight);
    if ((ncate==12||ncate==21)&&EF_e20_medium)
      m_looseESig->Fill(1,weight);
    if ((ncate==13||ncate==31)&&TriggerMu)
      m_looseMSig->Fill(1,weight);
  }
  //C1 : SELECTION OF EVENT WITH PRIMARY VERTEX 
  GetSelBranch(entry,1);
  if (!AtLeastOneGoodPrimaryVertex(4)) return kTRUE;
  m_loose->Fill(2,weight);
  if (EF_e20_medium)
    m_looseE->Fill(2,weight);
  if (TriggerMu)
    m_looseM->Fill(2,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(2,weight);
    if ((ncate==12||ncate==21)&&EF_e20_medium)
      m_looseESig->Fill(2,weight);
    if ((ncate==13||ncate==31)&&TriggerMu)
      m_looseMSig->Fill(2,weight);
  }

  //Before choosing lepton and jets
  //Veto events with at least a jet with track pointing to the LAr Hole
  GetSelBranch(entry,2);
  //double sumptjetInHole = VetoTrackJetInHole();
  //b_SumPtJetInHole = sumptjetInHole;
  //b_METHole = topMET_et*cos(topMET_phi)/1000 - MET_Truth_NonInt_etx/1000;
  
  //###### SELECTION OF LEPTON ######
  GetSelBranch(entry,3);
  vector<Particle> goodLepton;

  //###### SELECTION OF ELECTRON ######
  vector<TLorentzVector> goodE;
  vector<int> IndGoodE;
  vector<TLorentzVector> goodM;

  for (unsigned int i = 0; i < el_cl_pt->size(); i++) {
    if(GoodElectron(i)) {
      TLorentzVector v4e;
      fprintf(testptSEl2,"%10.2f\n",m_ptElSm);

      v4e.SetPtEtaPhiM(m_ptElSm,el_tracketa->at(i),el_trackphi->at(i),0);
      TLorentzVector v4ecl;
      v4ecl.SetPtEtaPhiM(el_cl_pt->at(i),el_cl_eta->at(i),el_cl_phi->at(i),0);

      int epol = 0;
      if (doMatch) epol = el_truth_barcode->at(i);
      Particle e(v4e,11,1,epol);
      goodLepton.push_back(e);
      goodE.push_back(v4e);
      IndGoodE.push_back(i);
    }
  }

  //###### SELECTION OF MUON ######
  vector<int> indGoodMuon;
  for (unsigned int i = 0; i < mu_pt->size(); i++) {
    if(GoodMuon(i)) {
      indGoodMuon.push_back(i);
      TLorentzVector v4m;

      fprintf(testptSMu2,"%10.2f\n",m_ptMuSm);

      v4m.SetPtEtaPhiM(m_ptMuSm,mu_eta->at(i),mu_phi->at(i),0);

      int pol = 0;
      if (doMatch) pol = mu_truth_barcode->at(i);
      Particle mu(v4m,13,1,pol);
      goodLepton.push_back(mu);
      goodM.push_back(v4m);
    }
  }

  double wSFemu = SFemu(goodE,goodM,IndGoodE);
  if (UseTool3)
    weight *= wSFemu;
  hwSFemu->Fill(wSFemu);
  b_SFemu = wSFemu;         

  //if (wSFemu!=1)
  //cout << "wSFemu = " << wSFemu << endl;

  //C2 : At least 1 good electron or 1 good muon
  if (goodE.size()<1 && goodM.size()<1) return kTRUE;
  m_loose->Fill(3,weight);
  if (goodE.size()>=1 && EF_e20_medium)
    m_looseE->Fill(3,weight);
  if (goodM.size()>=1 && TriggerMu)
    m_looseM->Fill(3,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(3,weight);
    if ((ncate==12||ncate==21)&&goodE.size()>=1 && EF_e20_medium)
      m_looseESig->Fill(3,weight);
    if ((ncate==13||ncate==31)&&goodM.size()>=1 && TriggerMu)
      m_looseMSig->Fill(3,weight);
  }
  //Print EventNumber list to a file
  //if (goodE.size()>=1 && EF_e20_medium)
  if (goodE.size()>=1)
    fprintf(EvtListC2,"%10i %10i\n",EventNumber,0);
  if (goodM.size()>=1)
    fprintf(EvtListC2,"%10i %10i\n",0,EventNumber);

  //C3 : Exactly 1 good electron or exactly 1 good muon
  if (goodE.size()!=1 && goodM.size()!=1) return kTRUE;
  m_loose->Fill(4,weight);
  if (goodE.size()==1 && EF_e20_medium)
    m_looseE->Fill(4,weight);
  if (goodM.size()==1 && TriggerMu)
    m_looseM->Fill(4,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(4,weight);
    if ((ncate==12||ncate==21)&&goodE.size()>=1 && EF_e20_medium)
      m_looseESig->Fill(4,weight);
    if ((ncate==13||ncate==31)&&goodM.size()>=1 && TriggerMu)
      m_looseMSig->Fill(4,weight);
  }

  //Print EventNumber list to a file
  //if (goodE.size()==1 && EF_e20_medium)
  if (goodE.size()==1)
    fprintf(EvtListC3,"%10i %10i\n",EventNumber,0);
  if (goodM.size()==1)
    fprintf(EvtListC3,"%10i %10i\n",0,EventNumber);

  //C4 : Exactly 0 good electron or exactly 0 good muon
  //requirement only one lep at this step
  if (goodE.size()!=0 && goodM.size()!=0) return kTRUE;
  m_loose->Fill(5,weight);
  if (goodE.size()==1 && EF_e20_medium)
    m_looseE->Fill(5,weight);
  if (goodM.size()==1 && TriggerMu)
    m_looseM->Fill(5,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(5,weight);
    if ((ncate==12||ncate==21)&&goodE.size()>=1 && EF_e20_medium)
      m_looseESig->Fill(5,weight);
    if ((ncate==13||ncate==31)&&goodM.size()>=1 && TriggerMu)
      m_looseMSig->Fill(5,weight);
  }

  //Print EventNumber list to a file
  //if (goodE.size()==1 && EF_e20_medium)
  if (goodE.size()==1)
    fprintf(EvtListC4,"%10i %10i\n",EventNumber,0);
  if (goodM.size()==1)
    fprintf(EvtListC4,"%10i %10i\n",0,EventNumber);

  //######## TRIGGERS ########
  GetSelBranch(entry,4);

  //C5 : the chosen lepton matches the trigger
  int ige = (IndGoodE.size() > 0) ? IndGoodE.at(0) : 0;
  if (!MatchTrigger(ige,goodE,goodM)) return kTRUE;
  m_loose->Fill(6,weight);
  if (goodE.size()==1)
    m_looseE->Fill(6,weight);
 if (goodM.size()==1)
    m_looseM->Fill(6,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(6,weight);
    if ((ncate==12||ncate==21)&&goodE.size()==1)
      m_looseESig->Fill(6,weight);
    if ((ncate==13||ncate==31)&&goodM.size()==1)
      m_looseMSig->Fill(6,weight);
  }

  //Print EventNumber list to a file
  if (goodE.size()==1)
    fprintf(EvtListC5,"%10i %10i\n",EventNumber,0);
  if (goodM.size()==1)
    fprintf(EvtListC5,"%10i %10i\n",0,EventNumber);

  //C6 : remove events tagged as e-mu overlap
  if (emuOverlap(IndGoodE)) return kTRUE;
  m_loose->Fill(7,weight);
  if (goodE.size()==1){
    isEleEvent = true;   
    m_looseE->Fill(7,weight);
  }
  if (goodM.size()==1)
    m_looseM->Fill(7,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(7,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(7,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(7,weight);
  }

  //C7 : Remove the event if there is one LooseBad jet with pt > 20 GeV and E > 0
  GetSelBranch(entry,5);
  vector<TLorentzVector> vjet;
  for (int i = 0; i < jet_n; i++) {
    TLorentzVector v4j;
    v4j.SetPtEtaPhiM(jet_pt->at(i),jet_eta->at(i),jet_phi->at(i),jet_m->at(i));
    vjet.push_back(v4j);
  }
  if (RealData && OneLooseBadJet(20e3,0)) return kTRUE;
  int hasBadJet = 0;
  if (!MET_Util.isJetSafe(vjet,jet_BCH_CORR_JET,jet_BCH_CORR_CELL,RealData,0.0))
    hasBadJet = 1;
  b_hasBadJet = hasBadJet;
  //if (m_ApplyHoleVeto && hasLArHole && hasBadJet) return kTRUE;

  m_loose->Fill(8,weight);
  if (goodE.size()==1)
    m_looseE->Fill(8,weight);
  if (goodM.size()==1)
    m_looseM->Fill(8,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(8,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(8,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(8,weight);
  }

  //C8 : MET
  std::vector<Particle> Wlep;
  Particle lp = goodLepton.at(0);
  TLorentzVector v4l = lp.vector4();

  double pTl = v4l.Pt();
  double pXl = v4l.Vect().X();
  double pYl = v4l.Vect().Y();
  double pZl = v4l.Vect().Z();
  double El  = v4l.E();

  double EXm = topMET_etx;
  double EYm = topMET_ety; 
  double ETm = topMET_et;
  double METPhi = topMET_phi;
  
  double mTW = sqrt(fabs(2*pTl*ETm*(1-cos(v4l.Vect().Phi() - METPhi))));

  if (goodE.size()==1) {
    if (ETm < 35e3) return kTRUE;
    m_looseE->Fill(9,weight);
    if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseESig->Fill(9,weight);
  } 
  if (goodM.size()==1){ 
    if (ETm < 20e3) return kTRUE;
    m_looseM->Fill(9,weight);
    if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseMSig->Fill(9,weight);
  }
  m_loose->Fill(9,weight);
  if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseSig->Fill(9,weight);

  //C9 : W transverse mass
  if (goodE.size()==1) {
    if ((mTW + ETm) < 60e3) return kTRUE;
    //if (mTW < 25e3) return kTRUE;
    m_looseE->Fill(10,weight);
    if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseESig->Fill(10,weight);
  }

  if (goodM.size()==1) {
    if ((ETm + mTW) < 60e3) return kTRUE;
    m_looseM->Fill(10,weight);
  if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseMSig->Fill(10,weight);  
  }

  m_loose->Fill(10,weight);
  if (ncate==12||ncate==13||ncate==21||ncate==31) m_looseSig->Fill(10,weight);

  //Before selecting jets, look if there is a muon coming from jet
  std::vector<std::pair<TLorentzVector,TLorentzVector> > MuonJetPairs = AddMuonJet(RealData,m_smearMC,indGoodMuon);
  //cout << "Number of pair muon and jet that overlap " << MuonJetPairs.size() << endl;

  vector<TLorentzVector> vMuon, vJet;
  for (unsigned int i = 0; i < MuonJetPairs.size(); i++){
    pair<TLorentzVector,TLorentzVector> MuonJet = MuonJetPairs.at(i);
    vMuon.push_back(MuonJet.first);
    vJet.push_back(MuonJet.second);
  }

  //SELECTION OF JETS
  vector<Particle> jets, goodJets;
  vector<TLorentzVector> bjets;
  vector<int> IndGoodJets;
  bool hasbtagged = false;
  double sv0Max = -999., sv0SubMax = -999.; 
  //vector<double> sv0GoodJets;
  for (int i = 0; i < jet_n; i++) {
    TLorentzVector v4j;
    v4j.SetPtEtaPhiM(jet_pt->at(i),jet_eta->at(i),jet_phi->at(i),jet_m->at(i));

    if (vJet.size()>0){
      for (unsigned int iJ = 0; iJ < vJet.size(); iJ++){
	TLorentzVector vJ = vJet.at(iJ);
	if (v4j==vJ) v4j += vMuon.at(iJ);
      }
    }

    jet_pt->at(i) = v4j.Pt();
    jet_E->at(i)  = v4j.E();

    if(GoodJets(i)) {
      double sv0 = jet_flavor_weight_SV0->at(i);
      //overlap removal jets-electron
      bool overlap_elejet = false;
      if (goodE.size()==1){
	TLorentzVector v4e = goodE.at(0);

	double dRej = v4j.DeltaR(v4e);
	if (dRej < 0.2) overlap_elejet = true;
      }

      if (!overlap_elejet){
	int flavor = -1;
	if (!RealData)
	  flavor = abs(jet_flavor_truth_label->at(i));
	Particle q(v4j,2,1,flavor);
	goodJets.push_back(q);
	IndGoodJets.push_back(i);

	// a small btag study
	int lab = 0;
	if (flavor == 15)     lab = 1;
	else if (flavor == 4) lab = 2;
	else if (flavor == 5) lab = 3;
	m_sv0[lab]->Fill(max(-19.99,min(sv0,39.99)));
	float ip3d = jet_flavor_weight_Comb->at(i);
	m_ip3dsv1[lab]->Fill(max(-19.99,min(ip3d,39.99)));
	float combnn = jet_flavor_weight_JetFitterCOMBNN->at(i);
	//
	//if (sv0 > 5.85){
	if (combnn > 2.0){
	  hasbtagged = true;
	  bjets.push_back(v4j);	  
	}
	if (sv0 > sv0Max) {
	  sv0SubMax = sv0Max;
	  sv0Max = sv0;
	}
      }
    }
  }

  m_weiSV0Max[0]->Fill(sv0Max,weight);

  bool askbtagged = false;

  double Ptmax = 1e-30;
  vector<TLorentzVector> vGoodJets;
  vector<int> vGoodJetsFlav;

  unsigned int nGoodJets = goodJets.size();

  TLorentzVector leadingjet;
  double HT = 0., sumptjets = 0., sumptjetsmu = 0.;
  for (unsigned int i = 0; i < nGoodJets; i++){
    //for (unsigned int i = 0; i < 4; i++){
    Particle parj = goodJets.at(i);
    vGoodJetsFlav.push_back(parj.POL());
    TLorentzVector v4 = parj.vector4();
    vGoodJets.push_back(v4);
    double Pt = v4.Pt();
    HT += Pt;
    sumptjets += Pt;
    if (Pt>Ptmax){
      Ptmax = Pt;
      leadingjet = v4;
    }
  }

  if (!isEleEvent) sumptjetsmu = sumptjets;

  for (unsigned int i = 0; i < goodLepton.size(); i++){
    double pTlep = goodLepton.at(i).vector4().Pt();
    HT += pTlep;
    if (!isEleEvent) sumptjetsmu += pTlep;
  }
  HT += topMET_et;

  //pt leading jet : C15 : add for u4
  if (Ptmax < 60e3) return kTRUE;
  m_loose->Fill(11,weight);
  if (goodE.size()==1) m_looseE->Fill(11,weight);
  if (goodM.size()==1) m_looseM->Fill(11,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(11,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(11,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(11,weight);
  }

  //need to fix it here
  double wSFb = SFbtag(IndGoodJets);
  if (UseTool4)
    weight *= wSFb;
  hwSFb->Fill(wSFb);
  b_SFbtag = wSFb;        
  b_WeiEvent = weight;

  m_loose->Fill(12,weight);
  if (goodE.size()==1) m_looseE->Fill(12,weight);
  if (goodM.size()==1) m_looseM->Fill(12,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(12,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(12,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(12,weight);
  }

  //Histogram for number of good jets
  //Step 1 : now rewei
  m_njets->Fill(goodJets.size(),weight);
  if (goodE.size()==1 && EF_e20_medium) m_njetsE->Fill(goodJets.size(),weight);
  if (goodM.size()==1 && TriggerMu) m_njetsM->Fill(goodJets.size(),weight);

  if (hasbtagged){
    m_njetsB->Fill(goodJets.size(),weight);
    if (goodE.size()==1 && EF_e20_medium) m_njetsBE->Fill(goodJets.size(),weight); 
    if (goodM.size()==1 && TriggerMu) m_njetsBM->Fill(goodJets.size(),weight);
  }

  //cout << "PUwei = " << PUwei << endl;

  //Step2 : *PUwei
  m_njets->Fill(goodJets.size()+50,weight*PUwei);
  if (goodE.size()==1 && EF_e20_medium) m_njetsE->Fill(goodJets.size()+50,weight*PUwei);
  if (goodM.size()==1 && TriggerMu) m_njetsM->Fill(goodJets.size()+50,weight*PUwei);

  if (hasbtagged){
    m_njetsB->Fill(goodJets.size()+50,weight*PUwei);
    if (goodE.size()==1 && EF_e20_medium) m_njetsBE->Fill(goodJets.size()+50,weight*PUwei);
    if (goodM.size()==1 && TriggerMu) m_njetsBM->Fill(goodJets.size()+50,weight*PUwei);
  }

  //Step3 " *wSFemu (emu)
  m_njets->Fill(goodJets.size()+100,weight*PUwei*wSFemu);
  if (goodE.size()==1 && EF_e20_medium) m_njetsE->Fill(goodJets.size()+100,weight*PUwei*wSFemu);
  if (goodM.size()==1 && TriggerMu) m_njetsM->Fill(goodJets.size()+100,weight*PUwei*wSFemu);

  if (hasbtagged){
    m_njetsB->Fill(goodJets.size()+100,weight*PUwei*wSFemu);
    if (goodE.size()==1 && EF_e20_medium) m_njetsBE->Fill(goodJets.size()+100,weight*PUwei*wSFemu);
    if (goodM.size()==1 && TriggerMu) m_njetsBM->Fill(goodJets.size()+100,weight*PUwei*wSFemu);
  }

  //Step3 " *wSFb 
  m_njets->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFb);
  if (goodE.size()==1 && EF_e20_medium) m_njetsE->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFb);
  if (goodM.size()==1 && TriggerMu) m_njetsM->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFb);

  if (hasbtagged){
    m_njetsB->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFemu);
    if (goodE.size()==1 && EF_e20_medium) m_njetsBE->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFb);
    if (goodM.size()==1 && TriggerMu) m_njetsBM->Fill(goodJets.size()+150,weight*PUwei*wSFemu*wSFb);
  }

  if (hasbtagged){
    if (goodE.size()==1 && EF_e20_medium)
      EvtListC14e << EventNumber << " " << goodJets.size() << endl;
    if (goodM.size()==1)
      EvtListC14m << EventNumber << " " << goodJets.size() << endl;
  }

  //C10 : at least 2 good jets
  if (goodJets.size()<2) return kTRUE;
  m_loose->Fill(13,weight);
  if (goodE.size()==1) m_looseE->Fill(13,weight);
  if (goodM.size()==1) m_looseM->Fill(13,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(13,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(13,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(13,weight);
  }

  //C11 : at least 3 good jets
  if (goodJets.size()<3) return kTRUE;
  m_loose->Fill(14,weight);
  if (goodE.size()==1) m_looseE->Fill(14,weight);
  if (goodM.size()==1) m_looseM->Fill(14,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(14,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(14,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(14,weight);
  }

  //C12 : at least 4 good jets
  if (goodJets.size()<4) return kTRUE;
  m_loose->Fill(15,weight);
  if (isEleEvent)
    m_looseE->Fill(15,weight);
  else m_looseM->Fill(15,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(15,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(15,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(15,weight);
  }

  //C13 : at least 4 good jets and one b-tagged
  if (!hasbtagged) return kTRUE;
  askbtagged = true;
  m_loose->Fill(16,weight);
  if (isEleEvent)
    m_looseE->Fill(16,weight);
  else m_looseM->Fill(16,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(16,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(16,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(16,weight);
  }

  //C14 : Verify the LAr Error : Normally this cut do nothing on MC
  if (larError!=0) return kTRUE;
  m_loose->Fill(17,weight);
  if (isEleEvent)
    m_looseE->Fill(17,weight);
  else m_looseM->Fill(17,weight);

  if (ncate==12||ncate==13||ncate==21||ncate==31){
    m_looseSig->Fill(17,weight);
    if (goodE.size()==1)
      m_looseESig->Fill(17,weight);
    if (goodM.size()==1)
      m_looseMSig->Fill(17,weight);
  }

  m_hfor[1]->Fill(top_hfor_type,weight);

  //################ END OF ALL SELECTIONS #################
  //Check the distance between the good jets
  for (unsigned int i = 0; i < nGoodJets; i++) {
    TLorentzVector vi = vGoodJets.at(i);
    for (unsigned int j = i + 1; j < nGoodJets; j++) {
      TLorentzVector vj = vGoodJets.at(j);
      double dR2goodjets = vi.DeltaR(vj);
      m_dR2goodjets->Fill(dR2goodjets,weight);
    }
  }

  //Francois' idea 6May 2011
  //Distribution mjj(bestW)
  double dMmin1 = 1e30;
  double mjjBest = 0.;
  for (unsigned int i = 0; i < nGoodJets; i++){
    TLorentzVector vj1 = vGoodJets.at(i);
    for (unsigned int j = 0; j < nGoodJets; j++){
      TLorentzVector vj2 = vGoodJets.at(j);
      if (vj2==vj1) continue;
      TLorentzVector vjj = vj1 + vj2;
      double mjj = vjj.M()/1000.;
      double dM = fabs(mjj - 80.4);
      if (dM < dMmin1){
	dMmin1 = dM;
	mjjBest = mjj;
      }
    }
  }

  m_mjjBest->Fill(mjjBest,weight);

  //Distribution of mjj(second bestW :-))
  double dMmin2 = 1e30;
  double mjjSecond = 0.;
  for (unsigned int i = 0; i < nGoodJets; i++){
    TLorentzVector vj1 = vGoodJets.at(i);
    for (unsigned int j = 0; j < nGoodJets; j++){
      TLorentzVector vj2 = vGoodJets.at(j);
      if (vj2==vj1) continue;
      TLorentzVector vjj = vj1 + vj2;
      double mjj = vjj.M()/1000.;
      double dM = fabs(mjj - 80.4);
      if ((dM < dMmin2) && (dM > dMmin1)){
	dMmin2 = dM;
	mjjSecond = mjj;
      }
    }
  }
  
  m_mjjSecond->Fill(mjjSecond,weight);

  iEvtSel++;
  for (int i = 0; i < 41; i++){
    double mjjM = 80.4 - i;
    double mjjP = 80.4 + i;

    if (mjjBest > mjjM && mjjBest < mjjP){
      Nevt[i]++;
    }
  }

  double dm1min = 1e30, dm2min = 1e30, dm3min = 1e30;
  double mjj1 = 0., mjj2 = 0., mjj3 = 0.;
  for (unsigned int i = 0; i < nGoodJets; i++){
    TLorentzVector vj1 = vGoodJets.at(i);
    for (unsigned int j = 0; j < nGoodJets; j++){
      TLorentzVector vj2 = vGoodJets.at(j);
      if (vj2==vj1) continue;
      TLorentzVector vjj = vj1 + vj2;
      double mjj = vjj.M()/1000.;
      double dm1 = fabs(mjj - 80);
      double dm2 = fabs(mjj - 60);
      double dm3 = fabs(mjj - 100);
      if (dm1 < dm1min){
	dm1min = dm1;
	mjj1 = mjj;
      }
      if (dm2 < dm2min){
	dm2min = dm2;
	mjj2 = mjj;
      }
      if (dm3 < dm3min){
	dm3min = dm3;
	mjj3 = mjj;
      }
    }
  }

  if (dm1min < dm2min && dm1min < dm3min)
    m_mjj1->Fill(mjj1,weight);

  if (dm2min < dm1min && dm2min < dm3min)
    m_mjj2->Fill(mjj2,weight);

  if (dm3min < dm1min && dm3min < dm2min)
    m_mjj3->Fill(mjj3,weight);  

  /*
  bool isMuonJet = false;
  TLorentzVector vMuonJet;
  TLorentzVector vJetOverlapNotGoodMuon;
  
  //On 14March : look at the muon in the e channel
  //TH1F *m_ptmuon_echannel, *m_dRmujets_echannel;
  if (isEleEvent) {
    for (unsigned int i = 0; i < mu_pt->size(); i++) {
      TLorentzVector v4m;
      v4m.SetPtEtaPhiM(mu_pt->at(i),mu_eta->at(i),mu_phi->at(i),0);

      double ptmu = mu_pt->at(i);
      m_ptmuon_echannel->Fill(ptmu/1000.,weight);

      for (unsigned int igj = 0; igj < nGoodJets; igj++) {
	TLorentzVector vgjet = vGoodJets.at(igj);
	double dRmugoodjets_echannel = v4m.DeltaR(vgjet);
	TLorentzVector vmugjet = v4m + vgjet;

	m_dRmu_goodjets_echannel->Fill(dRmugoodjets_echannel,weight);
	m_ptmu_goodjets_echannel->Fill(vmugjet.Pt()/1000,weight);
	m_etamu_goodjets_echannel->Fill(vmugjet.Eta(),weight);

	if (dRmugoodjets_echannel < 0.4) {
	  isMuonJet = true;
	  vMuonJet = v4m;
	  vJetOverlapNotGoodMuon = vgjet;
	}
      }

      for (int j = 0; j < jet_n; j++) {
	if (GoodJets(j)) continue;
	if (jet_pt->at(j) <= 0) continue;
	TLorentzVector v4j;
	v4j.SetPtEtaPhiM(jet_pt->at(j),jet_eta->at(j),jet_phi->at(j),jet_m->at(j));    

	double dRmunotgoodjets_echannel = v4m.DeltaR(v4j);
	if (dRmunotgoodjets_echannel < 0.4) { //in case this muon is close to a jet, may be the sum of the two vectors could give a good jet so i want to check now !!
	  TLorentzVector vmujet = v4m + v4j;
	  m_dRmu_notgoodjets_echannel->Fill(dRmunotgoodjets_echannel,weight);
	  m_ptmu_notgoodjets_echannel->Fill(vmujet.Pt()/1000,weight);
	  m_etamu_notgoodjets_echannel->Fill(vmujet.Eta(),weight);
	}
      }
    }
  }
  */ // comment for 26 August

  // #####################    START THE FIRST ANALYSIS    ####################### 
  // #### The Naive analysis : no fit, test all combinations,         ###########
  // #### take the one minimizing the difference between the two tops ###########  

  //###### RECONSTRUCT the LEPTONIC W ######
  double k0  = 80.4*80.4e6 + 2.*(pXl*EXm+pYl*EYm);
  double a0  = 4.*pTl*pTl;
  double b0  = -4.*k0*pZl;
  double c0  = 4.*El*El*ETm*ETm-k0*k0;

  double delta0 = b0*b0 - 4.*a0*c0;

  if (delta0 < 0) delta0 = 0;

  double pZmis1 = (-b0 + sqrt(delta0))/2./a0;
  double pZmis2 = (-b0 - sqrt(delta0))/2./a0;
  TVector3 pmis31(EXm, EYm, pZmis1);
  TVector3 pmis32(EXm, EYm, pZmis2);
  
  TLorentzVector pmis41(pmis31,pmis31.Mag());
  TLorentzVector pmis42(pmis32,pmis32.Mag());
  
  TLorentzVector vWl1 = v4l + pmis41;
  TLorentzVector vWl2 = v4l + pmis42;

  m_mWlep[0]->Fill(vWl1.M()/1000.,weight);
  m_mWlep[0]->Fill(vWl2.M()/1000.,weight);  
  m_EWlep[0]->Fill(vWl1.E()/1000.,weight);
  m_EWlep[0]->Fill(vWl2.E()/1000.,weight);

  if (isEleEvent) {
    m_mWlep[3]->Fill(vWl1.M()/1000.,weight);
    m_mWlep[3]->Fill(vWl2.M()/1000.,weight);  
    m_EWlep[3]->Fill(vWl1.E()/1000.,weight);
    m_EWlep[3]->Fill(vWl2.E()/1000.,weight);
  } else {
    m_mWlep[6]->Fill(vWl1.M()/1000.,weight);
    m_mWlep[6]->Fill(vWl2.M()/1000.,weight);  
    m_EWlep[6]->Fill(vWl1.E()/1000.,weight);
    m_EWlep[6]->Fill(vWl2.E()/1000.,weight);
  }

  std::set<short> setC; setC.insert(0);
  
  Particle W1(vWl1,24,2,0);
  W1.lepC(setC);
  Particle W2(vWl2,24,2,0);
  W2.lepC(setC);
  
  Wlep.push_back(W1);
  Wlep.push_back(W2);
  
  //###### RECONSTRUCT the corresponding HADRONIC W ######
  vector<Particle> Whad;
  //10 March
  double dRWhadj = 0.;
  double mlepjet = 0.;
  double pTWhad = 0.;
  double mjjj = 0.;

  for (unsigned int iq = 0; iq < nGoodJets; iq++) {
    Particle q1 = goodJets.at(iq);
    TLorentzVector v4q1 = q1.vector4();
    
    for (unsigned int jq = iq+1; jq < nGoodJets; jq++) {
      Particle q2 = goodJets.at(jq);
      TLorentzVector v4q2 = q2.vector4();

      TLorentzVector v4Wh = v4q1 + v4q2;

      if (fabs(v4Wh.M()-80.4e3)>20e3) continue;

      pTWhad = v4Wh.Pt();

      Particle Wh(v4Wh,24,2,0);
      std::set<short> setC1; setC1.insert(iq); setC1.insert(jq);
      Wh.jetC(setC1);//this is to know the constituents of hadronic W
      Whad.push_back(Wh);
      
    }//END of <for (int jq = iq+1; jq < goodJets.size(); jq++)>
  }//END of <for (int iq = 0; iq < goodJets.size(); iq++)>
  
  if (Whad.size()>0) {
    m_loose->Fill(100,weight);
    if (isEleEvent) m_looseE->Fill(100,weight);
    else m_looseM->Fill(100,weight);
    
    //############### HADRONIC TOP ###############
    vector<Particle> tophad;
    for (unsigned int iw = 0; iw < Whad.size(); iw++){
      Particle Wh = Whad.at(iw);
      TLorentzVector v4Wh = Wh.vector4();
      std::set<short> JetsFromWind = Wh.jetC();
      std::set<short>::iterator itE = JetsFromWind.end();
      for (unsigned int kq = 0; kq < nGoodJets; kq++){
        Particle q = goodJets.at(kq);
        TLorentzVector v4q = q.vector4();

        //we have to check the constituents of the W
        //that we've just reconstructed to make sure that
        //we don't use the same that we already used
        std::set<short>::iterator it = JetsFromWind.find(kq);
        if (it != itE) continue;
        TLorentzVector v4tophad = v4Wh + v4q;
	dRWhadj = v4Wh.DeltaR(v4q);

        Particle thad(v4tophad,8,2,0);
        std::set<short> constit;
        if (m_debug) cout << "Adding jet of index " << kq << " to hadronic top"<< endl;
        constit.insert((short)kq);
        std::set<short>::iterator kt = JetsFromWind.begin();
        for (; kt != itE; kt++) {
          if (m_debug) cout << "Adding jet (from W) of index " << *kt << " to hadronic top"<< endl;
          constit.insert(*kt);
        }
        thad.jetC(constit);
        
        tophad.push_back(thad);
      }//END of <for (int kq = 0; kq < nq; kq ++)>
    }//END of <for (iw = 0; iw < Whad.size(); iw++)>

    if (tophad.size()>0) {
      m_loose->Fill(101,weight);
      if (isEleEvent) m_looseE->Fill(101,weight);
      else m_looseM->Fill(101,weight);
      
      for (unsigned int i = 0; i < Whad.size(); i++){
        Particle w = Whad.at(i);
        TLorentzVector vw = w.vector4();
        m_mWhad[0]->Fill(vw.M()/1000,weight);
	m_EWhad[0]->Fill(vw.M()/1000,weight);
	if (isEleEvent) {
	  m_mWhad[3]->Fill(vw.M()/1000,weight);
	  m_EWhad[3]->Fill(vw.M()/1000,weight);
	} else {
	  m_mWhad[6]->Fill(vw.M()/1000,weight);
	  m_EWhad[6]->Fill(vw.M()/1000,weight);
	}
      }
      
      //###########################################
      // The leptonic top and the best combination
      std::vector<Particle> toplep;
      std::vector<std::pair<Particle,Particle> > toppair;
      for (unsigned int iw = 0; iw < Wlep.size(); iw++){
        Particle Wl = Wlep.at(iw);
        TLorentzVector v4Wl = Wl.vector4();
        
        for (unsigned int kq = 0; kq < nGoodJets; kq++){
          Particle q = goodJets.at(kq);
          TLorentzVector v4q = q.vector4();
          
          for (unsigned int itt = 0; itt < tophad.size(); itt++){
            Particle thad = tophad.at(itt);
            std::set<short>::iterator itE = thad.jetC().end();
            
            //we have to check if the quark is already contained 
            //in reconstruction de mass of hadronic top or not 
            std::set<short>::iterator it = thad.jetC().find(kq);
            if (it != itE) continue;
            
            TLorentzVector v4toplepcdd = v4Wl + v4q;
            Particle tlep(v4toplepcdd,8,2,0);
            std::set<short> lepc; lepc.insert(*Wl.lepC().begin());
            tlep.lepC(lepc);
            std::set<short> jetc; jetc.insert(kq);
            tlep.jetC(jetc);
            
            toplep.push_back(tlep);
            
            toppair.push_back(std::make_pair<Particle,Particle>(thad,tlep));
            
          }//END of <for (unsigned int itt = 0; itt < tophad.size(); itt++)>
        }//END of <for (int kq = 0; kq < nq; kq++)>
      }//END of <for (unsigned int iw = 0; iw < Wlep.size(); iw++)>

      int indGoodPair = - 1;  
      double dmmin = 1e9;
      double dm1 = 0;
      for (unsigned int ipair = 0; ipair < toppair.size(); ipair++){
        Particle toph = toppair.at(ipair).first;
        Particle topl = toppair.at(ipair).second;
        double dm  = fabs(toph.vector4().M()-topl.vector4().M())/1000.;
        double dm11 = (toph.vector4().M()-topl.vector4().M())/1000.;
        m_dmt[0]->Fill(dm,weight);
	if (isEleEvent) m_dmt[1]->Fill(dm,weight);
	else m_dmt[2]->Fill(dm,weight);

        if (dm < dmmin){
          dmmin = dm;
          indGoodPair = ipair;
          dm1 = dm11;
        }
      }
      
      m_dmmin[0]->Fill(dm1,weight);
      if (isEleEvent) m_dmmin[1]->Fill(dm1,weight);
      else m_dmmin[2]->Fill(dm1,weight);
      
      if (indGoodPair >= 0) {   // Should be impossible
        m_loose->Fill(102,weight);
	if (isEleEvent) m_looseE->Fill(102,weight);
	else m_looseM->Fill(102,weight);
        
        Particle gtoplep = toppair.at(indGoodPair).second;
        Particle gtophad = toppair.at(indGoodPair).first;
        
        double mthg = gtophad.vector4().M();
        double mtlg = gtoplep.vector4().M();
	mjjj = mthg;
        m_mtophad[0]->Fill(mthg/1e3,weight);
        m_mtoplep[0]->Fill(mtlg/1e3,weight);
	if (isEleEvent) {
	  m_mtophad[3]->Fill(mthg/1e3,weight);
	  m_mtoplep[3]->Fill(mtlg/1e3,weight);
	} else {
	  m_mtophad[6]->Fill(mthg/1e3,weight);
	  m_mtoplep[6]->Fill(mtlg/1e3,weight);
	}
      }//
    }//if (tophad.size()>0)
  }//if (Whad.size()>0)

  //Fill new histo on 10 March

  //Add histos on 10 March
  vector<TLorentzVector> Evtpars;
  double Ptmax2 = 1e-30;
  for (unsigned int i = 0; i < nGoodJets; i++){
    double Pt = vGoodJets.at(i).Pt();
    if (Pt>Ptmax2 && Pt<Ptmax)
      Ptmax2 = Pt;
  }  

  double dphiminMETjet = 1e30;
  for (unsigned int i = 0; i < nGoodJets; i++){
    Evtpars.push_back(vGoodJets.at(i));
    double dphi = vGoodJets.at(i).Phi() - topMET_phi;
    if (dphi < dphiminMETjet)
      dphiminMETjet = dphi;

    mlepjet = (goodLepton.at(0).vector4() + vGoodJets.at(i)).M();
    double dRlj = goodLepton.at(0).vector4().DeltaR(vGoodJets.at(i));
    m_dRlepjet[0]->Fill(dRlj,weight);
    if (isEleEvent) m_dRlepjet[1]->Fill(dRlj,weight);
    else m_dRlepjet[2]->Fill(dRlj,weight);
  }
  Evtpars.push_back(goodLepton.at(0).vector4());

  TVector2 vTlep = goodLepton.at(0).vector4().Vect().XYvector();
  TVector2 vTneu(topMET_etx,topMET_ety);
  TVector2 pTWlep = vTlep + vTneu;
  
  EventShape eshape(Evtpars,2.);

  int indexHistoChi2Bar = 0;
  if (isEleEvent) indexHistoChi2Bar = 3;
  else indexHistoChi2Bar = 6;

  m_sphericity[indexHistoChi2Bar]->Fill(eshape.sphericity(),weight);
  m_sphericityT[indexHistoChi2Bar]->Fill(eshape.sphericityT(),weight);
  m_aplanarity[indexHistoChi2Bar]->Fill(eshape.aplanarity(),weight);

  m_HT[indexHistoChi2Bar]->Fill(HT/1000.,weight); 
  m_sumptjets[indexHistoChi2Bar]->Fill(sumptjets/1000.,weight);
  m_sumptjetsmu[indexHistoChi2Bar]->Fill(sumptjetsmu/1000.,weight);
  m_ET[indexHistoChi2Bar]->Fill(topMET_sumet/1000.,weight);
  m_MET[indexHistoChi2Bar]->Fill(topMET_et/1000.,weight); 
  m_phiMET[indexHistoChi2Bar]->Fill(topMET_phi,weight);
  m_PTsubleadingjet[indexHistoChi2Bar]->Fill(Ptmax2/1e3,weight);
  m_PTleadingjet[indexHistoChi2Bar]->Fill(Ptmax/1e3,weight);
  m_MTlepneu[indexHistoChi2Bar]->Fill(mTW/1000.,weight);

  m_dphiminMETjet[indexHistoChi2Bar]->Fill(dphiminMETjet,weight);
  m_pTWlep[indexHistoChi2Bar]->Fill(pTWlep.Mod()/1000.,weight);

  m_mjjj[indexHistoChi2Bar]->Fill(mjjj/1000.,weight);
  m_dRWhadj[indexHistoChi2Bar]->Fill(dRWhadj,weight);
  m_pTWhad[indexHistoChi2Bar]->Fill(pTWhad/1000.,weight);
  m_mlepjet[indexHistoChi2Bar]->Fill(mlepjet/1000.,weight);

  m_twoDim_HT_mjjj[indexHistoChi2Bar]->Fill(HT/1000.,mjjj/1000.,weight);

  //TagTop172
  TagTop172(Wlep,vGoodJets,bjets,askbtagged);

  //########################################################
  //##############         STAR CHI2       #################
  //########################################################
  bool doChi2Ana = true;
  if (!doChi2Ana) return kTRUE;

  //Here I have to add this condition cause my chi2 method is happy 
  //just in the case I have one Lepton (e or mu)
  //if (goodepLepton.size()!=1) return kTRUE;
  
  //#### CHOOSE THE COMBINATIONS OF 4 JETS IN GOODJETS TO PUT IN CHI2 #####
  vector<vector<int> > indjets = choose4jets(vGoodJets);
  vector<double> chi2values;

  //Initialize the values
  int iindex = 0;
  int iconfigmin = 0;
  double chi2minmin = 1e30;
  double m2minmin = 0., m3minmin = 0., mWhadminmin = 0., mWlepminmin = 0., 
    tEbqqminmin = 0., tEq1Wminmin = 0., tEq2Wminmin = 0., tEbenminmin = 0., 
    tEeminmin   = 0., tmWhadminmin = 0., tmWlepminmin = 0., tdmt2minmin  = 0.;
  double Elepmin = -1;
  double Eqrkmin[4] = {-1,-1,-1,-1};

  TLorentzVector vthminmin, vtlminmin;
  TLorentzVector vnfitmin;
  TLorentzVector vWhadminmin, vWlepminmin;
  double mintmtminmin = 0;

  //int mWgoodcombi = 0;

  for (unsigned int id=0; id < indjets.size(); id++){
    
    //##########  Start calculating chi2  ##########
    double chi2min = 1e30;
    int nconfig = 24;
    int iconfig = 0;
    double m2min = 0, m3min = 0, mWhadmin = 0, mWlepmin = 0;
    TLorentzVector vthmin, vtlmin;
    TLorentzVector vnfit;
    TLorentzVector vWhadmin, vWlepmin;
    double tEbqqmin = 0., tEq1Wmin = 0., tEq2Wmin = 0., tEbenmin = 0., tEemin   = 0., 
      tmWhadmin = 0., tmWlepmin = 0., tdmt2min  = 0.;
    double mintmtmin = 0;

    vector<TLorentzVector> vecpars2;
    vector<int> index = indjets.at(id);
    //cout << "index size : " << index.size() << endl;

    vector<TLorentzVector> vJetsUseForChi2;
    for (unsigned int j=0; j < index.size(); j++){
      TLorentzVector vp4 = vGoodJets.at(index.at(j));
      vJetsUseForChi2.push_back(vp4);
    }
    
    //in case we ask for b-tagging
    //we have to force the bjets to be in the jets used to calculate chi2 first
    //here we don't care about the order yet
    vector<int> bjetsInd;
    vector<int> bjetsInd34;
    vector<TVector2> IndSV0jets34;
    //set<pair<double,double> > IndSV0;
    bool threebjets4GoodJets = false;
    if (askbtagged){
      bool bjetusedforchi2 = false;
      for (unsigned int i = 0; i < vJetsUseForChi2.size(); i++){
	TLorentzVector vj = vJetsUseForChi2.at(i);
	for (unsigned int j = 0; j < bjets.size(); j++){
	  TLorentzVector vb = bjets.at(j);
	  if (vj==vb) {
	    bjetusedforchi2 = true;
	    bjetsInd.push_back(i);
	  }
	}
      }
      if (!bjetusedforchi2) continue;

      if (vGoodJets.size()==4 && bjetsInd.size()==3){
	threebjets4GoodJets = true;
	TVector2 IndSV0;
	for (int inew = 0; inew < jet_n; inew++) {
	  TLorentzVector vj1;
	  vj1.SetPtEtaPhiM(jet_pt->at(inew),jet_eta->at(inew),jet_phi->at(inew),jet_m->at(inew));
	  double SV0j = jet_flavor_weight_SV0->at(inew);
	  for (unsigned int jnew = 0; jnew < vJetsUseForChi2.size(); jnew++){
	    TLorentzVector vj2 = vJetsUseForChi2.at(jnew);
	    if (vj1==vj2) {
	      IndSV0.Set(jnew,SV0j);
	      IndSV0jets34.push_back(IndSV0);
	      //IndSV0.insert(pair<double,double> (jnew,SV0j));
	    }
	  }
	}
	sort(IndSV0jets34.begin(),IndSV0jets34.end(),sortdecreaseSV0);
	bjetsInd34.push_back((int)IndSV0jets34.at(0).X());
	bjetsInd34.push_back((int)IndSV0jets34.at(1).X());
      }
    }//end of (askbtagged)
    
    /*for (unsigned int j=0; j < index.size(); j++){
      if (PRINT) cout << "index : " << index.at(j) << endl;
      TLorentzVector vp4 = vGoodJets.at(index.at(j));
      if (isMuonJet && vp4 == vJetOverlapNotGoodMuon) 
	vp4 += vMuonJet;
      vecpars2.push_back(vp4);
    }
    vecpars2.push_back(goodLepton.at(0).vector4());
    */ //comment for 26 August
    
    for (unsigned int j=0; j < index.size(); j++){
      if (PRINT) cout << "index : " << index.at(j) << endl;
      TLorentzVector vp4 = vGoodJets.at(index.at(j));
      vecpars2.push_back(vp4);
    }
    vecpars2.push_back(goodLepton.at(0).vector4());   

    //##############################################################
    //  Run on 24 possible configurations of the chosen combination
    for (int i = 0; i < nconfig; i++){
      int indq1W = q1W[i] - 1;
      int indq2W = q2W[i] - 1;

      bool bjetinW = false;
      if (threebjets4GoodJets) {
	for (unsigned int ib = 0; ib < bjetsInd34.size(); ib++) {
	  int indb = bjetsInd34.at(ib);
	  if (indb==indq1W||indb==indq2W)
	    bjetinW = true;
	}
      } else {
	for (unsigned int ib = 0; ib < bjetsInd.size(); ib++) {
	  int indb = bjetsInd.at(ib);
	  if (indb==indq1W||indb==indq2W)
	    bjetinW = true;
	}
      }

      if (bjetinW) continue;

      //CALL config function
      configfunction(vecpars2,topMET_etx,topMET_ety,i,true);  
      chi2class result = chi2calculation(i);

      double minimum = result.chi2();
      double bestE1 = result.ene().at(0);
      double bestE2 = result.ene().at(1);
      double bestE3 = result.ene().at(2);
      double bestE4 = result.ene().at(3);
      double bestE5 = result.ene().at(4);
      
      vector<double> neutrino = neutrinosolution(bestE1,bestE2,bestE3,bestE4,bestE5);

      double Eneu = neutrino.at(0);
      double costhetabenn = neutrino.at(1);
      double METx = neutrino.at(2);
      double METy = neutrino.at(3);
      double METz = neutrino.at(4);
      TVector3 vneu3fit(METx,METy,METz);

      TLorentzVector vneufit(vneu3fit,Eneu);
      TVector3 v3eold(Tab[20],Tab[21],Tab[22]);

      double ke   = bestE5/Tab[23];
      TVector3 v3e_fit   = ke*v3eold;
      TLorentzVector vefit(v3e_fit,bestE5);

      double costhetane = v3e_fit*vneu3fit/v3e_fit.Mag()/vneu3fit.Mag();
   
      double mWlepfit = 0, mWhadfit = 0, m2 = 0, m3 = 0;
      TLorentzVector vth, vtl;
      TLorentzVector vWhad, vWlep;
      double mintmt = 0;

      if (mjet0) {
        mWlepfit = sqrt(2*Eneu*bestE5*(1-costhetane));
        mWhadfit = sqrt(2*bestE2*bestE3*(1-Tab[4]));
        m2 = sqrt(2*(bestE2*bestE3*(1-Tab[4]) + bestE2*bestE1*(1-Tab[5]) + bestE3*bestE1*(1-Tab[6])));
        m3 = sqrt(80400.*80400. + 2*(bestE4*bestE5*(1-Tab[8]) + Eneu*bestE4*(1-costhetabenn)));
      }
      else {//m_jet is not 0
        //Recalculate the masses with 4-vectors
        TVector3 v3bqqold(Tab[10],Tab[15],Tab[28]);
        TVector3 v3q1Wold(Tab[11],Tab[16],Tab[27]);
        TVector3 v3q2Wold(Tab[12],Tab[17],Tab[26]);
        TVector3 v3benold(Tab[13],Tab[18],Tab[24]);

        double kbqq = bestE1/Tab[0];
        double kq1W = bestE2/Tab[1];
        double kq2W = bestE3/Tab[2];
        double kben = bestE4/Tab[3];
  
        TVector3 v3bqq_fit = kbqq*v3bqqold;
        TVector3 v3q1W_fit = kq1W*v3q1Wold;
        TVector3 v3q2W_fit = kq2W*v3q2Wold;
        TVector3 v3ben_fit = kben*v3benold;

        TLorentzVector vbqqfit(v3bqq_fit,bestE1);
        TLorentzVector vq1Wfit(v3q1W_fit,bestE2);
        TLorentzVector vq2Wfit(v3q2W_fit,bestE3);
        TLorentzVector vbenfit(v3ben_fit,bestE4);

        vWlep = vefit + vneufit;
        vWhad = vq1Wfit + vq2Wfit;
        vth = vWhad + vbqqfit;
        vtl = vWlep + vbenfit;

        mWlepfit = vWlep.M();
        mWhadfit = vWhad.M();
        m2 = vth.M();//masse top hadronique
        m3 = vtl.M();//masse top leptonique

	double tmt1 = m2 - 172e3;
	double tmt2 = m3 - 172e3;

	if (fabs(tmt1) < fabs(tmt2))
	  mintmt = m2/1000;
	else
	  mintmt = m3/1000;
      }

      TLorentzVector vpair = vth + vtl;
      double pTtt = vpair.Pt()/1000;
      m_pTtttouschi2[0]->Fill(pTtt,weight);
      if (isEleEvent) m_pTtttouschi2[1]->Fill(pTtt,weight);
      else m_pTtttouschi2[2]->Fill(pTtt,weight);

      //if (pTtt > 100) minimum = 1e40;
      //int pTtt_int = (int)pTtt;
      //double chi2add = chi2[pTtt_int - 10];
      //minimum += 10*chi2add;

      double tEbqq = (Tab[0] - bestE1)/(ksigma*0.5)/sqrt(bestE1*1000);
      double tEq1W = (Tab[1] - bestE2)/(ksigma*0.5)/sqrt(bestE2*1000);
      double tEq2W = (Tab[2] - bestE3)/(ksigma*0.5)/sqrt(bestE3*1000);
      double tEben = (Tab[3] - bestE4)/(ksigma*0.5)/sqrt(bestE4*1000);
      double tEe   = (Tab[23] - bestE5)/(ksigma*0.1)/sqrt(bestE5*1000);
  
      double tmWhad = (mWhadfit - 80400)/sigmaW;
      double tmWlep = (mWlepfit - 80400)/sigmaW;
      double tdmt2  = (m2 - m3)/sigmadmt;


      //Compare this value of 
      if (minimum < chi2min){//min in 24 configurations
        chi2min = minimum;
        iconfig = i;
        m2min = m2;
        m3min = m3;
        mWhadmin = mWhadfit;
        mWlepmin = mWlepfit;
        tEbqqmin = tEbqq;
        tEq1Wmin = tEq1W;
        tEq2Wmin = tEq2W;
        tEbenmin = tEben;
        tEemin   = tEe;
        tmWhadmin = tmWhad;
        tmWlepmin = tmWlep;
        tdmt2min  = tdmt2;
        Elepmin   = bestE5;
        Eqrkmin[0] = bestE1;
        Eqrkmin[1] = bestE2;
        Eqrkmin[2] = bestE3;    
        Eqrkmin[3] = bestE4;
        vnfit = vneufit;
        vthmin = vth;
        vtlmin = vtl;
	vWhadmin = vWhad;
	vWlepmin = vWlep;
	mintmtmin = mintmt;
      }
    } // end loop over nconfig

    chi2values.push_back(chi2min);  

    if (chi2min < chi2minmin){//min in all the possible combinations of 4 jets
      iindex = id;
      iconfigmin = iconfig;
      chi2minmin = chi2min;
      m2minmin = m2min;
      m3minmin = m3min;
      mWhadminmin = mWhadmin;
      mWlepminmin = mWlepmin;

      tEbqqminmin = tEbqqmin*tEbqqmin;
      tEq1Wminmin = tEq1Wmin*tEq1Wmin;
      tEq2Wminmin = tEq2Wmin*tEq2Wmin;
      tEbenminmin = tEbenmin*tEbenmin;
      tEeminmin   = tEemin*tEemin;
      tmWhadminmin = tmWhadmin*tmWhadmin;
      tmWlepminmin = tmWlepmin*tmWlepmin;
      tdmt2minmin  = tdmt2min*tdmt2min;

      vnfitmin = vnfit;
      vthminmin = vthmin;
      vtlminmin = vtlmin;
      vWhadminmin = vWhadmin;
      vWlepminmin = vWlepmin;

      mintmtminmin = mintmtmin;
    }

  }//end of loop on indjets
  
  TLorentzVector vttpairchi2 = vthminmin + vtlminmin;

  //Choose a second value of chi2
  double dchimin = 1e30;
  double chi2second = 0;
  for (unsigned int i = 0; i < chi2values.size(); i++){
    double chi2find = chi2values.at(i);
    double dchi2 = chi2find - chi2minmin;
    if (dchi2 > 0 && dchi2 < dchimin){
      dchimin = dchi2;
      chi2second = chi2find;
    }
  }
  
  //add new histos
  //First take the combination of 4jets that gives chi2min
  vector<int> indexmin = indjets.at(iindex);
  vector<TLorentzVector> vindex;

  int LeadingJetsInChi2 = 0;
  for (unsigned int i = 0; i < indexmin.size(); i++){
    int id = indexmin.at(i);
    if (id==0||id==1||id==2||id==3) LeadingJetsInChi2++;
  }

  double pTtt = vttpairchi2.Pt()/1000.;

  b_mWfit  = vWhadminmin.M()/1000.;
  b_mWBest = mjjBest;
  b_LeadingJetsInChi2 = LeadingJetsInChi2;

  if (pTtt < 150 && LeadingJetsInChi2 < 4){
    m_mtophad_00->Fill(m2minmin/1000.,weight);
    m_twoDim_mjjj_HT_00->Fill(m2minmin/1000.,HT/1000.,weight);
  }
  if (pTtt > 150 && LeadingJetsInChi2 < 4){
    m_mtophad_10->Fill(m2minmin/1000.,weight);
    m_twoDim_mjjj_HT_10->Fill(m2minmin/1000.,HT/1000.,weight);
  }
  if (pTtt < 150 && LeadingJetsInChi2 == 4){
    m_mtophad_01->Fill(m2minmin/1000.,weight);
    m_twoDim_mjjj_HT_01->Fill(m2minmin/1000.,HT/1000.,weight);
    m_mWfit->Fill(vWhadminmin.M()/1000.,weight);
    m_mjjBest4leadingjets->Fill(mjjBest,weight);
  }
  if (pTtt > 150 && LeadingJetsInChi2 == 4){
    m_mtophad_11->Fill(m2minmin/1000.,weight);
    m_twoDim_mjjj_HT_11->Fill(m2minmin/1000.,HT/1000.,weight);
    m_mWfit->Fill(vWhadminmin.M()/1000.,weight);
    m_mjjBest4leadingjets->Fill(mjjBest,weight);
  }

  //cout << "test2" << endl;
  
  for (unsigned int i=0; i < indexmin.size(); i++){
    TLorentzVector vj = vGoodJets.at(indexmin.at(i)); 
    vindex.push_back(vj);
  }
  TLorentzVector jbqq = vindex.at(bqq[iconfigmin] - 1);
  TLorentzVector jq1W = vindex.at(q1W[iconfigmin] - 1);
  TLorentzVector jq2W = vindex.at(q2W[iconfigmin] - 1);
  TLorentzVector jben = vindex.at(ben[iconfigmin] - 1);
  vector<TLorentzVector> FourJetsChi2;
  FourJetsChi2.push_back(jbqq);
  FourJetsChi2.push_back(jq1W);
  FourJetsChi2.push_back(jq2W);
  FourJetsChi2.push_back(jben);

  //Flavour Chi2
  FillFlavourChi2(vGoodJets,goodJets,jbqq,jq1W,jq2W,jben);

  //Matrix Element
  //Init LHAPDF
  //double s = 4*Ep*Ep;

  double s = 4*35e5*35e5;

  double m      = vthminmin.M();
  double Et     = vthminmin.E();
  double Etbar  = vtlminmin.E();
  double pZt    = vthminmin.Z();
  double pZtbar = vtlminmin.Z();
  double pTt    = vthminmin.Pt();
  double m2T    = m*m + pTt*pTt;
  double mT     = sqrt(m2T);

  double yt    = (1./2)*log((Et+pZt)/(Et-pZt));
  double ytbar = (1./2)*log((Etbar+pZtbar)/(Etbar-pZtbar));
  double dy    = yt - ytbar;

  double x1 = (mT/sqrt(s))*(exp(yt)+exp(ytbar));
  double x2 = (mT/sqrt(s))*(exp(-yt)+exp(-ytbar));

  double weight_gg    = 0;
  double weight_qqbar = 0;

  /*
  double xf1_g = LHAPDF::xfx(3,x1,mT,0);
  if (xf1_g < 0) xf1_g = 0;
  double xf2_g = LHAPDF::xfx(3,x2,mT,0);
  if (xf2_g < 0) xf2_g = 0;

  //cout << "xf1_g = " << xf1_g << endl;
  
  double xf1_d    = LHAPDF::xfx(3,x1,mT,1);
  if (xf1_d < 0) xf1_d = 0;
  double xf1_dbar = LHAPDF::xfx(3,x1,mT,-1);
  if (xf1_dbar < 0) xf1_dbar = 0;
  double xf2_d    = LHAPDF::xfx(3,x2,mT,1);
  if (xf2_d < 0) xf2_d = 0;
  double xf2_dbar = LHAPDF::xfx(3,x2,mT,-1);
  if (xf2_dbar < 0) xf2_dbar = 0;
  
  double xf1_u    = LHAPDF::xfx(3,x1,mT,2);
  if (xf1_u < 0) xf1_u = 0;
  double xf1_ubar = LHAPDF::xfx(3,x1,mT,-2);
  if (xf1_ubar < 0) xf1_ubar = 0;
  double xf2_u    = LHAPDF::xfx(3,x2,mT,2);
  if (xf2_u < 0) xf2_u = 0;
  double xf2_ubar = LHAPDF::xfx(3,x2,mT,-2);
  if (xf2_ubar < 0) xf2_ubar = 0;
  
  double xf1_s    = LHAPDF::xfx(3,x1,mT,3);
  if (xf1_s < 0) xf1_s = 0;
  double xf2_sbar = LHAPDF::xfx(3,x2,mT,-3);
  if (xf2_sbar < 0) xf2_sbar = 0;
  
  double xf1_c    = LHAPDF::xfx(3,x1,mT,4);
  if (xf1_c < 0) xf1_c = 0;
  double xf2_cbar = LHAPDF::xfx(3,x2,mT,-4);
  if (xf2_cbar < 0) xf2_cbar = 0;

  double xf1_b    = LHAPDF::xfx(3,x1,mT,5);
  if (xf1_b < 0) xf1_b = 0;
  double xf2_bbar = LHAPDF::xfx(3,x2,mT,-5);
  if (xf2_bbar < 0) xf2_bbar = 0;
  
  weight_gg = xf1_g*xf2_g;
  weight_qqbar = xf1_d*xf2_dbar + xf1_dbar*xf2_d + xf1_u*xf2_ubar + xf1_ubar*xf2_u + xf1_s*xf2_sbar + xf1_c*xf2_cbar + xf1_b*xf2_bbar;
  */
  //cout << "weight_gg = " << weight_gg << endl;

  double pTt_renor = pTt;
  double coef_formT_2 = (1+pTt_renor*pTt_renor/m/m)*(1+pTt_renor*pTt_renor/m/m);

  //cout << "pTt_renor = " << pTt_renor << endl;
  //cout << "coef_formT_2 = " << coef_formT_2 << endl;

  double coef = 1./coef_formT_2/(1+cosh(dy))/(1+cosh(dy));
  double M2QQ = Mqqbar2(dy,m,m2T);
  double M2GG = Mgg2(dy,m,m2T);

  double dsigmaQQ = coef*weight_qqbar*M2QQ;
  double dsigmaGG = coef*weight_gg*M2GG;
  double dsigmatot = dsigmaQQ + dsigmaGG;

  //cout << "dsigmatot = " << dsigmatot << endl;
  
  double dsigmaQQ_log = log(dsigmaQQ);
  double dsigmaGG_log = log(dsigmaGG);
  double dsigmatot_log = log(dsigmatot);

  m_twoDim_Logdsigma_HT->Fill(dsigmatot_log,HT,weight);

  //cout << "dsigmatot_log = " << dsigmatot_log << endl;

  b_dsigmaQQ  = dsigmaQQ;
  b_dsigmaGG  = dsigmaGG;
  b_dsigmatot = dsigmatot;

  if (chi2minmin > 200) {
    mtophad_grandchi2[0]->Fill(m2minmin/1000.,weight);
    if (isEleEvent)     mtophad_grandchi2[1]->Fill(m2minmin/1000.,weight);
    else     mtophad_grandchi2[2]->Fill(m2minmin/1000.,weight);
  }

  //These plots are especially for chi2 methode 
  //so the indices are 0,1,2 for all,Ele,Mu
  chi2histo[0]->Fill(chi2minmin,weight);
  m_dmtchi2[0]->Fill((m2minmin-m3minmin)/1000.,weight);
  m_twoDim_m2min_chi2min[0]->Fill(m2minmin/1000,chi2minmin,weight);
  m_twoDim_mintmt_chi2[0]->Fill(mintmtminmin,chi2minmin,weight);
  m_twoDim_m2min_pTtophad[0]->Fill(m2minmin/1000,vthminmin.Pt()/1000,weight);
  m_pTtophad_all[0]->Fill(vthminmin.Pt()/1000,weight);
  m_pTttpairchi2[0]->Fill(vttpairchi2.Pt()/1000,weight);
  m_twoDim_m2min_pTttpairchi2[0]->Fill(m2minmin/1000,vttpairchi2.Pt()/1000,weight);

  if (isEleEvent) {
    chi2histo[1]->Fill(chi2minmin,weight);
    m_dmtchi2[1]->Fill((m2minmin-m3minmin)/1000.,weight);
    m_twoDim_m2min_chi2min[1]->Fill(m2minmin/1000,chi2minmin,weight);
    m_twoDim_mintmt_chi2[1]->Fill(mintmtminmin,chi2minmin,weight);
    m_twoDim_m2min_pTtophad[1]->Fill(m2minmin/1000,vthminmin.Pt()/1000,weight);
    m_pTtophad_all[1]->Fill(vthminmin.Pt()/1000,weight);
    m_pTttpairchi2[1]->Fill(vttpairchi2.Pt()/1000,weight);
    m_twoDim_m2min_pTttpairchi2[1]->Fill(m2minmin/1000,vttpairchi2.Pt()/1000,weight);
  } else {
    chi2histo[2]->Fill(chi2minmin,weight);
    m_dmtchi2[2]->Fill((m2minmin-m3minmin)/1000.,weight);
    m_twoDim_m2min_chi2min[2]->Fill(m2minmin/1000,chi2minmin,weight);
    m_twoDim_mintmt_chi2[2]->Fill(mintmtminmin,chi2minmin,weight);
    m_twoDim_m2min_pTtophad[2]->Fill(m2minmin/1000,vthminmin.Pt()/1000,weight);
    m_pTtophad_all[2]->Fill(vthminmin.Pt()/1000,weight);
    m_pTttpairchi2[2]->Fill(vttpairchi2.Pt()/1000,weight);
    m_twoDim_m2min_pTttpairchi2[2]->Fill(m2minmin/1000,vttpairchi2.Pt()/1000,weight);
  }

  //Fill the Delta of Elep et Eqrk
  m_DeltaElep[0]->Fill((Tab[23]-Elepmin)/1e3,weight);
  m_DeltaEqrk[0]->Fill((jbqq.T()-Eqrkmin[0])/1e3,weight);      
  m_DeltaEqrk[0]->Fill((jq1W.T()-Eqrkmin[1])/1e3,weight);    
  m_DeltaEqrk[0]->Fill((jq2W.T()-Eqrkmin[2])/1e3,weight);
  m_DeltaEqrk[0]->Fill((jben.T()-Eqrkmin[3])/1e3,weight);

  if (isEleEvent) {
    m_DeltaElep[1]->Fill((Tab[23]-Elepmin)/1e3,weight);
    m_DeltaEqrk[1]->Fill((jbqq.T()-Eqrkmin[0])/1e3,weight);      
    m_DeltaEqrk[1]->Fill((jq1W.T()-Eqrkmin[1])/1e3,weight);    
    m_DeltaEqrk[1]->Fill((jq2W.T()-Eqrkmin[2])/1e3,weight);
    m_DeltaEqrk[1]->Fill((jben.T()-Eqrkmin[3])/1e3,weight);
  } else {
    m_DeltaElep[2]->Fill((Tab[23]-Elepmin)/1e3,weight);
    m_DeltaEqrk[2]->Fill((jbqq.T()-Eqrkmin[0])/1e3,weight);      
    m_DeltaEqrk[2]->Fill((jq1W.T()-Eqrkmin[1])/1e3,weight);    
    m_DeltaEqrk[2]->Fill((jq2W.T()-Eqrkmin[2])/1e3,weight);
    m_DeltaEqrk[2]->Fill((jben.T()-Eqrkmin[3])/1e3,weight);
  }

  TLorentzVector vlep = goodLepton.at(0).vector4();
  TLorentzVector vneu;
  TLorentzVector vneu2;
  double etaNeu = 0., ptNeu = 0.;
  
  if (iconfigmin < 12){
    vneu = pmis41;
    vneu2 = pmis42;
    etaNeu = pmis41.Eta();
    ptNeu =  pmis41.Pt();
  }
  else {
    vneu = pmis42;
    vneu2 = pmis41;
    etaNeu = pmis42.Eta();
    ptNeu =  pmis42.Pt();
  }

  //Verify if there are two matched jets merged into same jets
  double mjetMax = -1e9;
  for (unsigned int i = 0; i < nGoodJets; i++){
    double mjet = vGoodJets.at(i).M();
    if (mjet > mjetMax){
      mjetMax = mjet;
    }
  }
  
  m_mjetMax->Fill(mjetMax/1000,weight);
  b_mjetMax = mjetMax/1000;
  
  m_category[1]->Fill(ncate,weight);
  
  //MATCH :: First compare to matching informations

  if ((!RealData)&&(doMatch)){
    
    //cout << "enter here??" << endl;

    b_catChi2 = ncate;

    //cout << "b_catChi2 = " << b_catChi2 << endl;

    TruthAna(weight,mcpars,resParsMC,isEleEvent,goodLepton.at(0),ncate,Ite,vGoodJets,FourJetsChi2,MatchedPars,vneu,vneu2,m2minmin,chi2minmin,chi2second);
  }//end of <compare to the matching infos

  std::vector<double> etas;
  etas.push_back(fabs(jbqq.Eta()));
  etas.push_back(fabs(jq1W.Eta()));
  etas.push_back(fabs(jq2W.Eta()));
  etas.push_back(fabs(jben.Eta()));
  etas.push_back(fabs(vlep.Eta()));
  etas.push_back(fabs(etaNeu));

  std::sort(etas.begin(),etas.end(),sortincreaseDouble);

  double dRqq_afterchi2 = jq1W.DeltaR(jq2W);

  double dRmineq = 1e30;
  for (unsigned int i = 0; i < vindex.size(); i++){
    double dR = goodLepton.at(0).vector4().DeltaR(vindex.at(i));
    if (dR<dRmineq)
      dRmineq = dR;
  }

  //Thrust 
  Thrust_func_collection obj;
  vector<double> ResThrust = obj.Calc_Thrust(vindex);
  double ThrustT = ResThrust.at(0);
  double ThrustTT = ResThrust.at(1);

  //Invariant mass of Wqq
  TLorentzVector vWqq = jq1W + jq2W;
  TLorentzVector vWen = vlep + vneu;

  //Vector of top and topbar
  TLorentzVector vthad = vWqq + jbqq;
  TLorentzVector vtlep = vWen + jben;
  TLorentzVector vtpair = vthad + vtlep;

  TVector3 thad_boost = vthad.BoostVector();
  TVector3 tlep_boost = vtlep.BoostVector();
  TVector3 tpair_boost = vtpair.BoostVector();
  TVector3 Wqq_boost  = vWqq.BoostVector();
  TVector3 Wen_boost  = vWen.BoostVector();

  //Mag of impulsions
  double PWqq = vWqq.Vect().Mag();
  double PWen = vWen.Vect().Mag();
  double P_thad = vthad.Vect().Mag();
  double P_tlep = vtlep.Vect().Mag();

  //Some more distributions in the lab
  //Costheta of tpair
  TVector3 vunit(0,0,1);
  TVector3 v3tpair = vtpair.Vect();
  double costpair_lab = v3tpair.Dot(vunit)/vunit.Mag()/v3tpair.Mag();
  
  //dRminbWqq
  double dRminbhadWqq = 1e3;
  double dRminblepWqq = 1e3;
  double dRq1Wbqq = jq1W.DeltaR(jbqq);
  if (dRq1Wbqq < dRminbhadWqq) dRminbhadWqq = dRq1Wbqq;
  double dRq2Wbqq = jq2W.DeltaR(jbqq);
  if (dRq2Wbqq < dRminbhadWqq) dRminbhadWqq = dRq2Wbqq;
  double dRq1Wben = jq1W.DeltaR(jben);
  if (dRq1Wben < dRminblepWqq) dRminblepWqq = dRq1Wben;
  double dRq2Wben = jq2W.DeltaR(jben);
  if (dRq2Wben < dRminblepWqq) dRminblepWqq = dRq2Wben;
  //dRbb
  double dRbb = jben.DeltaR(jbqq);
  //dRminNeuWqq
  double dRNeuq1W = vneu.DeltaR(jq1W);
  double dRNeuq2W = vneu.DeltaR(jq2W);
  double dRminNeuWqq = min(dRNeuq1W,dRNeuq2W);
  //dRminbNeu
  double dRbhadNeu = vneu.DeltaR(jbqq);
  double dRblepNeu = vneu.DeltaR(jben);
  double dRminbNeu = min(dRbhadNeu,dRblepNeu);

  //Direction of tlep in CM of top pair
  TLorentzVector vtlep_CMtpair = vtlep;
  vtlep_CMtpair.Boost(-tpair_boost);
  TVector3 v3tlep_CMtpair = vtlep_CMtpair.Vect();
  double cos_tlepCMtpair = v3tlep_CMtpair.Dot(tpair_boost)/tpair_boost.Mag()/v3tlep_CMtpair.Mag();

  TLorentzVector vWqq_boosted = vWqq;
  TLorentzVector vWen_boosted = vWen;
  vWqq_boosted.Boost(-thad_boost);
  vWen_boosted.Boost(-tlep_boost);
  TVector3 v3Wqq_boosted = vWqq_boosted.Vect();
  TVector3 v3Wen_boosted = vWen_boosted.Vect();

  //Costheta of the Ws with respect to the corresponding top boost vector
  double cosWqq = v3Wqq_boosted.Dot(thad_boost)/v3Wqq_boosted.Mag()/thad_boost.Mag();
  double cosWen = v3Wen_boosted.Dot(tlep_boost)/v3Wen_boosted.Mag()/tlep_boost.Mag();

  //Direction of the 2 quarks W in W rest frame with respect to the W-boost
  TLorentzVector jq1W_boosted = jq1W;
  TLorentzVector jq2W_boosted = jq2W;
  jq1W_boosted.Boost(-Wqq_boost);
  jq2W_boosted.Boost(-Wqq_boost);
  TVector3 v3jq1W_boosted = jq1W_boosted.Vect();
  TVector3 v3jq2W_boosted = jq2W_boosted.Vect();
  double cosqW1 = v3jq1W_boosted.Dot(Wqq_boost)/Wqq_boost.Mag()/v3jq1W_boosted.Mag();
  double cosqW2 = v3jq2W_boosted.Dot(Wqq_boost)/Wqq_boost.Mag()/v3jq2W_boosted.Mag();
  
  double cosq1CMw_topboost = v3jq1W_boosted.Dot(thad_boost)/thad_boost.Mag()/v3jq1W_boosted.Mag();
  double cosq2CMw_topboost = v3jq2W_boosted.Dot(thad_boost)/thad_boost.Mag()/v3jq2W_boosted.Mag();

  //Direction of charged lepton in CM of W
  TLorentzVector vlep_boosted1 = vlep;
  vlep_boosted1.Boost(-Wen_boost);
  TVector3   v3lep_boosted1 = vlep_boosted1.Vect();
  double cos_lepCMw = v3lep_boosted1.Dot(Wen_boost)/Wen_boost.Mag()/v3lep_boosted1.Mag();

  //Direction of the charged lepton in t' rest frame
  TLorentzVector vlep_boosted2 = vlep;
  vlep_boosted2.Boost(-tlep_boost);
  TVector3 v3lep_boosted2 = vlep_boosted2.Vect();
  //Direction of the charged lepton in t' rest frame
  double cos_lepCMt = v3lep_boosted2.Dot(tlep_boost)/tlep_boost.Mag()/v3lep_boosted2.Mag();

  //Direction of neutrino in t' rest frame
  TLorentzVector vneu_boosted = vneu;
  vneu_boosted.Boost(-tlep_boost);
  TVector3 v3neu_boosted = vneu_boosted.Vect();
  double cos_bneuCMt = v3neu_boosted.Dot(tlep_boost)/v3neu_boosted.Mag()/tlep_boost.Mag();

  //Direction between Wen and b in t' rest frame
  TLorentzVector vjben_boosted = jben;
  vjben_boosted.Boost(-tlep_boost);
  TVector3 v3jben_boosted = vjben_boosted.Vect();
  double cos_bWenCMt = v3Wen_boosted.Dot(v3jben_boosted)/v3Wen_boosted.Mag()/v3jben_boosted.Mag();

  //angle between the charged lepton and quark top
  double costheta_qtlep = v3lep_boosted2.Dot(v3jben_boosted)/v3lep_boosted2.Mag()/v3jben_boosted.Mag();

  m_x1[1]->Fill(x1,weight);
  m_x2[1]->Fill(x2,weight);

  m_Th1_Th2[1]->Fill(ThrustT,ThrustTT,weight);
  m_HT[1]->Fill(HT/1000.,weight); 
  m_sumptjets[1]->Fill(sumptjets/1000.,weight);
  m_sumptjetsmu[1]->Fill(sumptjetsmu/1000.,weight);
  m_ET[1]->Fill(topMET_sumet/1000.,weight);
  m_MET[1]->Fill(topMET_et/1000.,weight); 
  m_phiMET[1]->Fill(topMET_phi,weight);
  m_PTsubleadingjet[1]->Fill(Ptmax2/1e3,weight);
  m_PTleadingjet[1]->Fill(Ptmax/1e3,weight);
  m_MTlepneu[1]->Fill(mTW/1000.,weight);
  
  m_dphiminMETjet[1]->Fill(dphiminMETjet,weight);
  m_pTWlep[1]->Fill(pTWlep.Mod()/1000.,weight);
  
  m_mjjj[1]->Fill(mjjj/1000.,weight);
  m_dRWhadj[1]->Fill(dRWhadj,weight);
  m_pTWhad[1]->Fill(pTWhad/1000.,weight);
  m_mlepjet[1]->Fill(mlepjet/1000.,weight);
  m_twoDim_HT_mjjj[1]->Fill(HT/1000.,mjjj/1000.,weight);
  
  m_dsigmaQQ[1]->Fill(dsigmaQQ_log,weight);
  m_dsigmaGG[1]->Fill(dsigmaGG_log,weight);
  m_dsigmatot[1]->Fill(dsigmatot_log,weight); 
  m_yplus[1]->Fill(yt,weight);
  m_yminus[1]->Fill(ytbar,weight);
  m_yplus_yminus[1]->Fill(yt,ytbar,weight);
  m_pTtop[1]->Fill(pTt/1000.,weight);
  m_pTtop_dsigma[1]->Fill(dsigmatot,pTt/1000.,weight);

  m_mWhad[1]->Fill(mWhadminmin/1000.,weight);
  m_mWlep[1]->Fill(mWlepminmin/1000.,weight);
  m_sphericity[1]->Fill(eshape.sphericity(),weight);
  m_sphericityT[1]->Fill(eshape.sphericityT(),weight);
  m_aplanarity[1]->Fill(eshape.aplanarity(),weight);
  
  m_mtophad[1]->Fill(m2minmin/1000.,weight);
  m_mtoplep[1]->Fill(m3minmin/1000.,weight);
  m_EWhad[1]->Fill(vWhadminmin.E()/1000,weight);
  m_EWlep[1]->Fill(vWlepminmin.E()/1000,weight);
  m_pTtophad[1]->Fill(vthminmin.Pt()/1000,weight);
  m_pTtoplep[1]->Fill(vtlminmin.Pt()/1000,weight);
  m_pZtophad[1]->Fill(vthminmin.Z()/1000,weight);
  m_pZttbar[1]->Fill(vttpairchi2.Z()/1000,weight);
  m_etajbqq[1]->Fill(jbqq.Eta(),weight);
  m_etajq1W[1]->Fill(jq1W.Eta(),weight);
  m_etajq2W[1]->Fill(jq2W.Eta(),weight);
  m_etajben[1]->Fill(jben.Eta(),weight);
  m_etaLep[1]->Fill(vlep.Eta(),weight);
  m_etaNeu[1]->Fill(etaNeu,weight);
  m_etaMin[1]->Fill(etas.at(0),weight);  
  m_etaMax[1]->Fill(etas.at(5),weight);
  m_ptjbqq[1]->Fill(jbqq.Pt()/1000.,weight);
  m_ptjq1W[1]->Fill(jq1W.Pt()/1000.,weight);
  m_ptjq2W[1]->Fill(jq2W.Pt()/1000.,weight);
  m_ptjben[1]->Fill(jben.Pt()/1000.,weight);
  m_ptLep[1]->Fill(vlep.Pt()/1000.,weight);
  m_ptNeu[1]->Fill(ptNeu/1000.,weight);
  m_dRqqW[1]->Fill(dRqq_afterchi2,weight);
  m_dRmin_eq[1]->Fill(dRmineq,weight);

  m_ctet_qtlep[1]->Fill(costheta_qtlep,weight);
  m_cos_q1CMw_topboost[1]->Fill(cosq1CMw_topboost,weight);
  m_cos_q2CMw_topboost[1]->Fill(cosq2CMw_topboost,weight);
  m_cos_lepCMw[1]->Fill(cos_lepCMw,weight);
  m_cos_bWenCMt[1]->Fill(cos_bWenCMt,weight);
  m_cos_lepCMt[1]->Fill(cos_lepCMt,weight);
  m_cos_bneuCMt[1]->Fill(cos_bneuCMt,weight);
  m_cosWqq[1]->Fill(cosWqq,weight);
  m_cosWen[1]->Fill(cosWen,weight);
  m_cosqW1[1]->Fill(fabs(cosqW1),weight);
  m_cosqW2[1]->Fill(cosqW2,weight);

  m_PWqq[1]->Fill(PWqq/1000.,weight);
  m_PWen[1]->Fill(PWen/1000.,weight);
  m_P_thad[1]->Fill(P_thad/1000.,weight);
  m_P_tlep[1]->Fill(P_tlep/1000.,weight);
  m_costpair_lab[1]->Fill(costpair_lab,weight);
  m_dRminbNeu[1]->Fill(dRminbNeu,weight);
  m_dRminbhadWqq[1]->Fill(dRminbhadWqq,weight);
  m_dRminblepWqq[1]->Fill(dRminblepWqq,weight);
  m_dRbb[1]->Fill(dRbb,weight);
  m_dRminNeuWqq[1]->Fill(dRminNeuWqq,weight);
  m_cos_tlepCMtpair[1]->Fill(cos_tlepCMtpair,weight);
  m_ptWqq[1]->Fill(vWqq.Pt()/1000.,weight);
  m_ptWen[1]->Fill(vWen.Pt()/1000.,weight);

  if (isEleEvent) {
    m_x1[4]->Fill(x1,weight);
    m_x2[4]->Fill(x2,weight);
    
    m_Th1_Th2[4]->Fill(ThrustT,ThrustTT,weight);
    m_HT[4]->Fill(HT/1000.,weight); 
    m_sumptjets[4]->Fill(sumptjets/1000.,weight);
    m_sumptjetsmu[4]->Fill(sumptjetsmu/1000.,weight);
    m_ET[4]->Fill(topMET_sumet/1000.,weight);
    m_MET[4]->Fill(topMET_et/1000.,weight); 
    m_phiMET[4]->Fill(topMET_phi,weight);
    m_PTsubleadingjet[4]->Fill(Ptmax2/1e3,weight);
    m_PTleadingjet[4]->Fill(Ptmax/1e3,weight);
    m_MTlepneu[4]->Fill(mTW/1000.,weight);
    
    m_dphiminMETjet[4]->Fill(dphiminMETjet,weight);
    m_pTWlep[4]->Fill(pTWlep.Mod()/1000.,weight);
    
    m_mjjj[4]->Fill(mjjj/1000.,weight);
    m_dRWhadj[4]->Fill(dRWhadj,weight);
    m_pTWhad[4]->Fill(pTWhad/1000.,weight);
    m_mlepjet[4]->Fill(mlepjet/1000.,weight);
    m_twoDim_HT_mjjj[4]->Fill(HT/1000.,mjjj/1000.,weight);
    
    m_dsigmaQQ[4]->Fill(dsigmaQQ_log,weight);
    m_dsigmaGG[4]->Fill(dsigmaGG_log,weight);
    m_dsigmatot[4]->Fill(dsigmatot_log,weight); 
    m_yplus[4]->Fill(yt,weight);
    m_yminus[4]->Fill(ytbar,weight);
    m_yplus_yminus[4]->Fill(yt,ytbar,weight);
    m_pTtop[4]->Fill(pTt/1000.,weight);
    m_pTtop_dsigma[4]->Fill(dsigmatot,pTt/1000.,weight);
    
    m_mWhad[4]->Fill(mWhadminmin/1000.,weight);
    m_mWlep[4]->Fill(mWlepminmin/1000.,weight);
    m_sphericity[4]->Fill(eshape.sphericity(),weight);
    m_sphericityT[4]->Fill(eshape.sphericityT(),weight);
    m_aplanarity[4]->Fill(eshape.aplanarity(),weight);
    
    m_mtophad[4]->Fill(m2minmin/1000.,weight);
    m_mtoplep[4]->Fill(m3minmin/1000.,weight);
    m_EWhad[4]->Fill(vWhadminmin.E()/1000,weight);
    m_EWlep[4]->Fill(vWlepminmin.E()/1000,weight);
    m_pTtophad[4]->Fill(vthminmin.Pt()/1000,weight);
    m_pTtoplep[4]->Fill(vtlminmin.Pt()/1000,weight);
    m_pZtophad[4]->Fill(vthminmin.Z()/1000,weight);
    m_pZttbar[4]->Fill(vttpairchi2.Z()/1000,weight);
    m_etajbqq[4]->Fill(jbqq.Eta(),weight);
    m_etajq1W[4]->Fill(jq1W.Eta(),weight);
    m_etajq2W[4]->Fill(jq2W.Eta(),weight);
    m_etajben[4]->Fill(jben.Eta(),weight);
    m_etaLep[4]->Fill(vlep.Eta(),weight);
    m_etaNeu[4]->Fill(etaNeu,weight);
    m_etaMin[4]->Fill(etas.at(0),weight);  
    m_etaMax[4]->Fill(etas.at(5),weight);
    m_ptjbqq[4]->Fill(jbqq.Pt()/1000.,weight);
    m_ptjq1W[4]->Fill(jq1W.Pt()/1000.,weight);
    m_ptjq2W[4]->Fill(jq2W.Pt()/1000.,weight);
    m_ptjben[4]->Fill(jben.Pt()/1000.,weight);
    m_ptLep[4]->Fill(vlep.Pt()/1000.,weight);
    m_ptNeu[4]->Fill(ptNeu/1000.,weight);
    m_dRqqW[4]->Fill(dRqq_afterchi2,weight);
    m_dRmin_eq[4]->Fill(dRmineq,weight);

    m_ctet_qtlep[4]->Fill(costheta_qtlep,weight);
    m_cos_q1CMw_topboost[4]->Fill(cosq1CMw_topboost,weight);
    m_cos_q2CMw_topboost[4]->Fill(cosq2CMw_topboost,weight);
    m_cos_lepCMw[4]->Fill(cos_lepCMw,weight);
    m_cos_bWenCMt[4]->Fill(cos_bWenCMt,weight);
    m_cos_lepCMt[4]->Fill(cos_lepCMt,weight);
    m_cos_bneuCMt[4]->Fill(cos_bneuCMt,weight);
    m_cosWqq[4]->Fill(cosWqq,weight);
    m_cosWen[4]->Fill(cosWen,weight);
    m_cosqW1[4]->Fill(fabs(cosqW1),weight);
    m_cosqW2[4]->Fill(cosqW2,weight);

    m_PWqq[4]->Fill(PWqq/1000.,weight);
    m_PWen[4]->Fill(PWen/1000.,weight);
    m_P_thad[4]->Fill(P_thad/1000.,weight);
    m_P_tlep[4]->Fill(P_tlep/1000.,weight);
    m_costpair_lab[4]->Fill(costpair_lab,weight);
    m_dRminbNeu[4]->Fill(dRminbNeu,weight);
    m_dRminbhadWqq[4]->Fill(dRminbhadWqq,weight);
    m_dRminblepWqq[4]->Fill(dRminblepWqq,weight);
    m_dRbb[4]->Fill(dRbb,weight);
    m_dRminNeuWqq[4]->Fill(dRminNeuWqq,weight);
    m_cos_tlepCMtpair[4]->Fill(cos_tlepCMtpair,weight);
    m_ptWqq[4]->Fill(vWqq.Pt()/1000.,weight);
    m_ptWen[4]->Fill(vWen.Pt()/1000.,weight);
  }
  else {
    m_x1[7]->Fill(x1,weight);
    m_x2[7]->Fill(x2,weight);

    m_Th1_Th2[7]->Fill(ThrustT,ThrustTT,weight);
    m_HT[7]->Fill(HT/1000.,weight); 
    m_sumptjets[7]->Fill(sumptjets/1000.,weight);
    m_sumptjetsmu[7]->Fill(sumptjetsmu/1000.,weight);
    m_ET[7]->Fill(topMET_sumet/1000.,weight);
    m_MET[7]->Fill(topMET_et/1000.,weight); 
    m_phiMET[7]->Fill(topMET_phi,weight);
    m_PTsubleadingjet[7]->Fill(Ptmax2/1e3,weight);
    m_PTleadingjet[7]->Fill(Ptmax/1e3,weight);
    m_MTlepneu[7]->Fill(mTW/1000.,weight);
  
    m_dphiminMETjet[7]->Fill(dphiminMETjet,weight);
    m_pTWlep[7]->Fill(pTWlep.Mod()/1000.,weight);
  
    m_mjjj[7]->Fill(mjjj/1000.,weight);
    m_dRWhadj[7]->Fill(dRWhadj,weight);
    m_pTWhad[7]->Fill(pTWhad/1000.,weight);
    m_mlepjet[7]->Fill(mlepjet/1000.,weight);
    m_twoDim_HT_mjjj[7]->Fill(HT/1000.,mjjj/1000.,weight);
  
    m_dsigmaQQ[7]->Fill(dsigmaQQ_log,weight);
    m_dsigmaGG[7]->Fill(dsigmaGG_log,weight);
    m_dsigmatot[7]->Fill(dsigmatot_log,weight); 
    m_yplus[7]->Fill(yt,weight);
    m_yminus[7]->Fill(ytbar,weight);
    m_yplus_yminus[7]->Fill(yt,ytbar,weight);
    m_pTtop[7]->Fill(pTt/1000.,weight);
    m_pTtop_dsigma[7]->Fill(dsigmatot,pTt/1000.,weight);

    m_mWhad[7]->Fill(mWhadminmin/1000.,weight);
    m_mWlep[7]->Fill(mWlepminmin/1000.,weight);
    m_sphericity[7]->Fill(eshape.sphericity(),weight);
    m_sphericityT[7]->Fill(eshape.sphericityT(),weight);
    m_aplanarity[7]->Fill(eshape.aplanarity(),weight);
  
    m_mtophad[7]->Fill(m2minmin/1000.,weight);
    m_mtoplep[7]->Fill(m3minmin/1000.,weight);
    m_EWhad[7]->Fill(vWhadminmin.E()/1000,weight);
    m_EWlep[7]->Fill(vWlepminmin.E()/1000,weight);
    m_pTtophad[7]->Fill(vthminmin.Pt()/1000,weight);
    m_pTtoplep[7]->Fill(vtlminmin.Pt()/1000,weight);
    m_pZtophad[7]->Fill(vthminmin.Z()/1000,weight);
    m_pZttbar[7]->Fill(vttpairchi2.Z()/1000,weight);
    m_etajbqq[7]->Fill(jbqq.Eta(),weight);
    m_etajq1W[7]->Fill(jq1W.Eta(),weight);
    m_etajq2W[7]->Fill(jq2W.Eta(),weight);
    m_etajben[7]->Fill(jben.Eta(),weight);
    m_etaLep[7]->Fill(vlep.Eta(),weight);
    m_etaNeu[7]->Fill(etaNeu,weight);
    m_etaMin[7]->Fill(etas.at(0),weight);  
    m_etaMax[7]->Fill(etas.at(5),weight);
    m_ptjbqq[7]->Fill(jbqq.Pt()/1000.,weight);
    m_ptjq1W[7]->Fill(jq1W.Pt()/1000.,weight);
    m_ptjq2W[7]->Fill(jq2W.Pt()/1000.,weight);
    m_ptjben[7]->Fill(jben.Pt()/1000.,weight);
    m_ptLep[7]->Fill(vlep.Pt()/1000.,weight);
    m_ptNeu[7]->Fill(ptNeu/1000.,weight);
    m_dRqqW[7]->Fill(dRqq_afterchi2,weight);
    m_dRmin_eq[7]->Fill(dRmineq,weight);

    m_ctet_qtlep[7]->Fill(costheta_qtlep,weight);
    m_cos_q1CMw_topboost[7]->Fill(cosq1CMw_topboost,weight);
    m_cos_q2CMw_topboost[7]->Fill(cosq2CMw_topboost,weight);
    m_cos_lepCMw[7]->Fill(cos_lepCMw,weight);
    m_cos_bWenCMt[7]->Fill(cos_bWenCMt,weight);
    m_cos_lepCMt[7]->Fill(cos_lepCMt,weight);
    m_cos_bneuCMt[7]->Fill(cos_bneuCMt,weight);
    m_cosWqq[7]->Fill(cosWqq,weight);
    m_cosWen[7]->Fill(cosWen,weight);
    m_cosqW1[7]->Fill(fabs(cosqW1),weight);
    m_cosqW2[7]->Fill(cosqW2,weight);

    m_PWqq[7]->Fill(PWqq/1000.,weight);
    m_PWen[7]->Fill(PWen/1000.,weight);
    m_P_thad[7]->Fill(P_thad/1000.,weight);
    m_P_tlep[7]->Fill(P_tlep/1000.,weight);
    m_costpair_lab[7]->Fill(costpair_lab,weight);
    m_dRminbNeu[7]->Fill(dRminbNeu,weight);
    m_dRminbhadWqq[7]->Fill(dRminbhadWqq,weight);
    m_dRminblepWqq[7]->Fill(dRminblepWqq,weight);
    m_dRbb[7]->Fill(dRbb,weight);
    m_dRminNeuWqq[7]->Fill(dRminNeuWqq,weight);
    m_cos_tlepCMtpair[7]->Fill(cos_tlepCMtpair,weight);
    m_ptWqq[7]->Fill(vWqq.Pt()/1000.,weight);
    m_ptWen[7]->Fill(vWen.Pt()/1000.,weight);
  }
  if (cosWqq > -1.0 && cosWqq < -0.7) {
    m_cosWqqNEW[3]->Fill(cosWqq,weight);
    m_mtophadNEW[3]->Fill(m2minmin/1000.,weight);
  } else if (cosWqq > -0.7 && cosWqq < 0.7) {
    m_cosWqqNEW[4]->Fill(cosWqq,weight);
    m_mtophadNEW[4]->Fill(m2minmin/1000.,weight);    
  } else if (cosWqq > 0.7 && cosWqq < 1.0) {
    m_cosWqqNEW[5]->Fill(cosWqq,weight);
    m_mtophadNEW[5]->Fill(m2minmin/1000.,weight);    
  }
  
  //Make the categories for cuts
  //chi2 analyse
  int nHT = 0, ncosWqq = 0, ncosWen = 0, ndRqqW = 0, ndRminbhadqqW = 0;
  if (HT/1000. > 600) nHT = 1;
  else                nHT = 0;
  if (fabs(cosWqq) <= 0.8) ncosWqq = 1;
  else                     ncosWqq = 0;
  if (cosWen <= 0.8) ncosWen = 1;
  else               ncosWen = 0;
  if (dRqq_afterchi2 <= 1.5) ndRqqW = 1;
  else                       ndRqqW = 0;
  if (dRminbhadWqq < 2.6) ndRminbhadqqW = 1;
  else                    ndRminbhadqqW = 0;

  int nCutCat = nHT + 5*ncosWqq + 10*ncosWen + 20*ndRqqW + 40*ndRminbhadqqW;

  m_CutCat[1]->Fill(nCutCat,weight);

  //fill the histos on top for the case where mtophad is greater than 300GeV  
  if (m2minmin/1000 > 300) {

    if ((m2minmin/1000 > 300) && (HT/1000 > 800)){
      nHyperNasty++;
      //cout << "Hyper nasty Top with HT > 800 !!!" << endl;
      //cout << "EventNumber " << EventNumber << endl;
    }

    m_twoDim_Logdsigma_HT_nasty->Fill(dsigmatot_log,HT,weight);
  
    m_CutCat[2]->Fill(nCutCat,weight);
    m_sphericity[2]->Fill(eshape.sphericity(),weight);
    m_sphericityT[2]->Fill(eshape.sphericityT(),weight);
    m_aplanarity[2]->Fill(eshape.aplanarity(),weight);
    
    m_Th1_Th2[2]->Fill(ThrustT,ThrustTT,weight);
    
    m_HT[2]->Fill(HT/1000.,weight); 
    m_sumptjets[2]->Fill(sumptjets/1000.,weight);
    m_sumptjetsmu[2]->Fill(sumptjetsmu/1000.,weight);
    m_ET[2]->Fill(topMET_sumet/1000.,weight);
    m_MET[2]->Fill(topMET_et/1000.,weight); 
    m_phiMET[2]->Fill(topMET_phi,weight);
    m_PTsubleadingjet[2]->Fill(Ptmax2/1e3,weight);
    m_PTleadingjet[2]->Fill(Ptmax/1e3,weight);
    m_MTlepneu[2]->Fill(mTW/1000.,weight);
    
    m_dphiminMETjet[2]->Fill(dphiminMETjet,weight);
    m_pTWlep[2]->Fill(pTWlep.Mod()/1000.,weight);
    
    m_mjjj[2]->Fill(mjjj/1000.,weight);
    m_dRWhadj[2]->Fill(dRWhadj,weight);
    m_pTWhad[2]->Fill(pTWhad/1000.,weight);
    m_mlepjet[2]->Fill(mlepjet/1000.,weight);
    m_twoDim_HT_mjjj[2]->Fill(HT/1000.,mjjj/1000.,weight);

    m_mtophad[2]->Fill(m2minmin/1000.,weight);
    m_mtoplep[2]->Fill(m3minmin/1000.,weight);
    m_EWhad[2]->Fill(vWhadminmin.E()/1000,weight);
    m_EWlep[2]->Fill(vWlepminmin.E()/1000,weight);
    m_pTtophad[2]->Fill(vthminmin.Pt()/1000,weight);
    m_pTtoplep[2]->Fill(vtlminmin.Pt()/1000,weight);
    m_pZtophad[2]->Fill(vthminmin.Z()/1000,weight);
    m_pZttbar[2]->Fill(vttpairchi2.Z()/1000,weight);
    m_etajbqq[2]->Fill(jbqq.Eta(),weight);
    m_etajq1W[2]->Fill(jq1W.Eta(),weight);
    m_etajq2W[2]->Fill(jq2W.Eta(),weight);
    m_etajben[2]->Fill(jben.Eta(),weight);
    m_etaLep[2]->Fill(vlep.Eta(),weight);
    m_etaNeu[2]->Fill(etaNeu,weight);
    m_etaMin[2]->Fill(etas.at(0),weight);  
    m_etaMax[2]->Fill(etas.at(5),weight);
    m_ptjbqq[2]->Fill(jbqq.Pt()/1000.,weight);
    m_ptjq1W[2]->Fill(jq1W.Pt()/1000.,weight);
    m_ptjq2W[2]->Fill(jq2W.Pt()/1000.,weight);
    m_ptjben[2]->Fill(jben.Pt()/1000.,weight);
    m_ptLep[2]->Fill(vlep.Pt()/1000.,weight);
    m_ptNeu[2]->Fill(ptNeu/1000.,weight);
    m_dRqqW[2]->Fill(dRqq_afterchi2,weight);
    m_dRmin_eq[2]->Fill(dRmineq,weight);

    //element matrix 
    m_x1[2]->Fill(x1,weight);
    m_x2[2]->Fill(x2,weight);
    m_dsigmaQQ[2]->Fill(dsigmaQQ_log,weight);
    m_dsigmaGG[2]->Fill(dsigmaGG_log,weight);
    m_dsigmatot[2]->Fill(dsigmatot_log,weight); 
    m_yplus[2]->Fill(yt,weight);
    m_yminus[2]->Fill(ytbar,weight);
    m_yplus_yminus[2]->Fill(yt,ytbar,weight);
    m_pTtop[2]->Fill(pTt/1000.,weight);
    m_pTtop_dsigma[2]->Fill(dsigmatot,pTt/1000.,weight);

    m_ctet_qtlep[2]->Fill(costheta_qtlep,weight);
    m_cos_q1CMw_topboost[2]->Fill(cosq1CMw_topboost,weight);
    m_cos_q2CMw_topboost[2]->Fill(cosq2CMw_topboost,weight);
    m_cos_lepCMt[2]->Fill(cos_lepCMt,weight);
    m_cos_bWenCMt[2]->Fill(cos_bWenCMt,weight);
    m_cos_lepCMw[2]->Fill(cos_lepCMw,weight);
    m_cos_bneuCMt[2]->Fill(cos_bneuCMt,weight);
    m_cosWqq[2]->Fill(cosWqq,weight);
    m_cosWen[2]->Fill(cosWen,weight);
    m_cosqW1[2]->Fill(fabs(cosqW1),weight);
    m_cosqW2[2]->Fill(cosqW2,weight);

    m_PWqq[2]->Fill(PWqq/1000.,weight);
    m_PWen[2]->Fill(PWen/1000.,weight);
    m_P_thad[2]->Fill(P_thad/1000.,weight);
    m_P_tlep[2]->Fill(P_tlep/1000.,weight);
    m_costpair_lab[2]->Fill(costpair_lab,weight);
    m_dRminbNeu[2]->Fill(dRminbNeu,weight);
    m_dRminbhadWqq[2]->Fill(dRminbhadWqq,weight);
    m_dRminblepWqq[2]->Fill(dRminblepWqq,weight);
    
    m_dRbb[2]->Fill(dRbb,weight);
    m_dRminNeuWqq[2]->Fill(dRminNeuWqq,weight);
    m_cos_tlepCMtpair[2]->Fill(cos_tlepCMtpair,weight);
    m_ptWqq[2]->Fill(vWqq.Pt()/1000.,weight);
    m_ptWen[2]->Fill(vWen.Pt()/1000.,weight);

    if (isEleEvent) {
      m_CutCat[5]->Fill(nCutCat,weight);
      m_sphericity[5]->Fill(eshape.sphericity(),weight);
      m_sphericityT[5]->Fill(eshape.sphericityT(),weight);
      m_aplanarity[5]->Fill(eshape.aplanarity(),weight);

      m_Th1_Th2[5]->Fill(ThrustT,ThrustTT,weight);

      m_HT[5]->Fill(HT/1000.,weight); 
      m_sumptjets[5]->Fill(sumptjets/1000.,weight);
      m_sumptjetsmu[5]->Fill(sumptjetsmu/1000.,weight);
      m_ET[5]->Fill(topMET_sumet/1000.,weight);
      m_MET[5]->Fill(topMET_et/1000.,weight); 
      m_phiMET[5]->Fill(topMET_phi,weight);
      m_PTsubleadingjet[5]->Fill(Ptmax2/1e3,weight);
      m_PTleadingjet[5]->Fill(Ptmax/1e3,weight);
      m_MTlepneu[5]->Fill(mTW/1000.,weight);
    
      m_dphiminMETjet[5]->Fill(dphiminMETjet,weight);
      m_pTWlep[5]->Fill(pTWlep.Mod()/1000.,weight);
    
      m_mjjj[5]->Fill(mjjj/1000.,weight);
      m_dRWhadj[5]->Fill(dRWhadj,weight);
      m_pTWhad[5]->Fill(pTWhad/1000.,weight);
      m_mlepjet[5]->Fill(mlepjet/1000.,weight);
      m_twoDim_HT_mjjj[5]->Fill(HT/1000.,mjjj/1000.,weight);

      m_mtophad[5]->Fill(m2minmin/1000.,weight);
      m_mtoplep[5]->Fill(m3minmin/1000.,weight);
      m_EWhad[5]->Fill(vWhadminmin.E()/1000,weight);
      m_EWlep[5]->Fill(vWlepminmin.E()/1000,weight);
      m_pTtophad[5]->Fill(vthminmin.Pt()/1000,weight);
      m_pTtoplep[5]->Fill(vtlminmin.Pt()/1000,weight);
      m_pZtophad[5]->Fill(vthminmin.Z()/1000,weight);
      m_pZttbar[5]->Fill(vttpairchi2.Z()/1000,weight);
      m_etajbqq[5]->Fill(jbqq.Eta(),weight);
      m_etajq1W[5]->Fill(jq1W.Eta(),weight);
      m_etajq2W[5]->Fill(jq2W.Eta(),weight);
      m_etajben[5]->Fill(jben.Eta(),weight);
      m_etaLep[5]->Fill(vlep.Eta(),weight);
      m_etaNeu[5]->Fill(etaNeu,weight);
      m_etaMin[5]->Fill(etas.at(0),weight);  
      m_etaMax[5]->Fill(etas.at(5),weight);
      m_ptjbqq[5]->Fill(jbqq.Pt()/1000.,weight);
      m_ptjq1W[5]->Fill(jq1W.Pt()/1000.,weight);
      m_ptjq2W[5]->Fill(jq2W.Pt()/1000.,weight);
      m_ptjben[5]->Fill(jben.Pt()/1000.,weight);
      m_ptLep[5]->Fill(vlep.Pt()/1000.,weight);
      m_ptNeu[5]->Fill(ptNeu/1000.,weight);
      m_dRqqW[5]->Fill(dRqq_afterchi2,weight);
      m_dRmin_eq[5]->Fill(dRmineq,weight);

      //element matrix 
      m_x1[5]->Fill(x1,weight);
      m_x2[5]->Fill(x2,weight);
      m_dsigmaQQ[5]->Fill(dsigmaQQ_log,weight);
      m_dsigmaGG[5]->Fill(dsigmaGG_log,weight);
      m_dsigmatot[5]->Fill(dsigmatot_log,weight); 
      m_yplus[5]->Fill(yt,weight);
      m_yminus[5]->Fill(ytbar,weight);
      m_yplus_yminus[5]->Fill(yt,ytbar,weight);
      m_pTtop[5]->Fill(pTt/1000.,weight);
      m_pTtop_dsigma[5]->Fill(dsigmatot,pTt/1000.,weight);

      m_ctet_qtlep[5]->Fill(costheta_qtlep,weight);
      m_cos_q1CMw_topboost[5]->Fill(cosq1CMw_topboost,weight);
      m_cos_q2CMw_topboost[5]->Fill(cosq2CMw_topboost,weight);
      m_cos_lepCMt[5]->Fill(cos_lepCMt,weight);
      m_cos_bWenCMt[5]->Fill(cos_bWenCMt,weight);
      m_cos_lepCMw[5]->Fill(cos_lepCMw,weight);
      m_cos_bneuCMt[5]->Fill(cos_bneuCMt,weight);
      m_cosWqq[5]->Fill(cosWqq,weight);
      m_cosWen[5]->Fill(cosWen,weight);
      m_cosqW1[5]->Fill(fabs(cosqW1),weight);
      m_cosqW2[5]->Fill(cosqW2,weight);

      m_PWqq[5]->Fill(PWqq/1000.,weight);
      m_PWen[5]->Fill(PWen/1000.,weight);
      m_P_thad[5]->Fill(P_thad/1000.,weight);
      m_P_tlep[5]->Fill(P_tlep/1000.,weight);
      m_costpair_lab[5]->Fill(costpair_lab,weight);
      m_dRminbNeu[5]->Fill(dRminbNeu,weight);
      m_dRminbhadWqq[5]->Fill(dRminbhadWqq,weight);
      m_dRminblepWqq[5]->Fill(dRminblepWqq,weight);
    
      m_dRbb[5]->Fill(dRbb,weight);
      m_dRminNeuWqq[5]->Fill(dRminNeuWqq,weight);
      m_cos_tlepCMtpair[5]->Fill(cos_tlepCMtpair,weight);
      m_ptWqq[5]->Fill(vWqq.Pt()/1000.,weight);
      m_ptWen[5]->Fill(vWen.Pt()/1000.,weight);
    } else {
      m_CutCat[8]->Fill(nCutCat,weight);
      m_sphericity[8]->Fill(eshape.sphericity(),weight);
      m_sphericityT[8]->Fill(eshape.sphericityT(),weight);
      m_aplanarity[8]->Fill(eshape.aplanarity(),weight);

      m_Th1_Th2[8]->Fill(ThrustT,ThrustTT,weight);

      m_HT[8]->Fill(HT/1000.,weight); 
      m_sumptjets[8]->Fill(sumptjets/1000.,weight);
      m_sumptjetsmu[8]->Fill(sumptjetsmu/1000.,weight);
      m_ET[8]->Fill(topMET_sumet/1000.,weight);
      m_MET[8]->Fill(topMET_et/1000.,weight); 
      m_phiMET[8]->Fill(topMET_phi,weight);
      m_PTsubleadingjet[8]->Fill(Ptmax2/1e3,weight);
      m_PTleadingjet[8]->Fill(Ptmax/1e3,weight);
      m_MTlepneu[8]->Fill(mTW/1000.,weight);
    
      m_dphiminMETjet[8]->Fill(dphiminMETjet,weight);
      m_pTWlep[8]->Fill(pTWlep.Mod()/1000.,weight);
    
      m_mjjj[8]->Fill(mjjj/1000.,weight);
      m_dRWhadj[8]->Fill(dRWhadj,weight);
      m_pTWhad[8]->Fill(pTWhad/1000.,weight);
      m_mlepjet[8]->Fill(mlepjet/1000.,weight);
      m_twoDim_HT_mjjj[8]->Fill(HT/1000.,mjjj/1000.,weight);

      m_mtophad[8]->Fill(m2minmin/1000.,weight);
      m_mtoplep[8]->Fill(m3minmin/1000.,weight);
      m_EWhad[8]->Fill(vWhadminmin.E()/1000,weight);
      m_EWlep[8]->Fill(vWlepminmin.E()/1000,weight);
      m_pTtophad[8]->Fill(vthminmin.Pt()/1000,weight);
      m_pTtoplep[8]->Fill(vtlminmin.Pt()/1000,weight);
      m_pZtophad[8]->Fill(vthminmin.Z()/1000,weight);
      m_pZttbar[8]->Fill(vttpairchi2.Z()/1000,weight);
      m_etajbqq[8]->Fill(jbqq.Eta(),weight);
      m_etajq1W[8]->Fill(jq1W.Eta(),weight);
      m_etajq2W[8]->Fill(jq2W.Eta(),weight);
      m_etajben[8]->Fill(jben.Eta(),weight);
      m_etaLep[8]->Fill(vlep.Eta(),weight);
      m_etaNeu[8]->Fill(etaNeu,weight);
      m_etaMin[8]->Fill(etas.at(0),weight);  
      m_etaMax[8]->Fill(etas.at(5),weight);
      m_ptjbqq[8]->Fill(jbqq.Pt()/1000.,weight);
      m_ptjq1W[8]->Fill(jq1W.Pt()/1000.,weight);
      m_ptjq2W[8]->Fill(jq2W.Pt()/1000.,weight);
      m_ptjben[8]->Fill(jben.Pt()/1000.,weight);
      m_ptLep[8]->Fill(vlep.Pt()/1000.,weight);
      m_ptNeu[8]->Fill(ptNeu/1000.,weight);
      m_dRqqW[8]->Fill(dRqq_afterchi2,weight);
      m_dRmin_eq[8]->Fill(dRmineq,weight);

      //element matrix 
      m_x1[8]->Fill(x1,weight);
      m_x2[8]->Fill(x2,weight);
      m_dsigmaQQ[8]->Fill(dsigmaQQ_log,weight);
      m_dsigmaGG[8]->Fill(dsigmaGG_log,weight);
      m_dsigmatot[8]->Fill(dsigmatot_log,weight); 
      m_yplus[8]->Fill(yt,weight);
      m_yminus[8]->Fill(ytbar,weight);
      m_yplus_yminus[8]->Fill(yt,ytbar,weight);
      m_pTtop[8]->Fill(pTt/1000.,weight);
      m_pTtop_dsigma[8]->Fill(dsigmatot,pTt/1000.,weight);

      m_ctet_qtlep[8]->Fill(costheta_qtlep,weight);
      m_cos_q1CMw_topboost[8]->Fill(cosq1CMw_topboost,weight);
      m_cos_q2CMw_topboost[8]->Fill(cosq2CMw_topboost,weight);
      m_cos_lepCMt[8]->Fill(cos_lepCMt,weight);
      m_cos_bWenCMt[8]->Fill(cos_bWenCMt,weight);
      m_cos_lepCMw[8]->Fill(cos_lepCMw,weight);
      m_cos_bneuCMt[8]->Fill(cos_bneuCMt,weight);
      m_cosWqq[8]->Fill(cosWqq,weight);
      m_cosWen[8]->Fill(cosWen,weight);
      m_cosqW1[8]->Fill(fabs(cosqW1),weight);
      m_cosqW2[8]->Fill(cosqW2,weight);

      m_PWqq[8]->Fill(PWqq/1000.,weight);
      m_PWen[8]->Fill(PWen/1000.,weight);
      m_P_thad[8]->Fill(P_thad/1000.,weight);
      m_P_tlep[8]->Fill(P_tlep/1000.,weight);
      m_costpair_lab[8]->Fill(costpair_lab,weight);
      m_dRminbNeu[8]->Fill(dRminbNeu,weight);
      m_dRminbhadWqq[8]->Fill(dRminbhadWqq,weight);
      m_dRminblepWqq[8]->Fill(dRminblepWqq,weight);
    
      m_dRbb[8]->Fill(dRbb,weight);
      m_dRminNeuWqq[8]->Fill(dRminNeuWqq,weight);
      m_cos_tlepCMtpair[8]->Fill(cos_tlepCMtpair,weight);
      m_ptWqq[8]->Fill(vWqq.Pt()/1000.,weight);
      m_ptWen[8]->Fill(vWen.Pt()/1000.,weight);
    }

    if (cosWqq > -1.0 && cosWqq < -0.7) {
      m_cosWqqNEW[6]->Fill(cosWqq,weight);
      m_mtophadNEW[6]->Fill(m2minmin/1000.,weight);
    } else if (cosWqq > -0.7 && cosWqq < 0.7) {
      m_cosWqqNEW[7]->Fill(cosWqq,weight);
      m_mtophadNEW[7]->Fill(m2minmin/1000.,weight);    
    } else if (cosWqq > 0.7 && cosWqq < 1.0) {
      m_cosWqqNEW[8]->Fill(cosWqq,weight);
      m_mtophadNEW[8]->Fill(m2minmin/1000.,weight);    
    }
  }

  //Check the domination of the terms in chi2 
  double dEq  = (tEbqqminmin + tEq1Wminmin + tEq2Wminmin + tEbenminmin)/chi2minmin;
  double dEl  = tEeminmin/chi2minmin;
  double dmW = (tmWhadminmin + tmWlepminmin)/chi2minmin;
  double dmt  = tdmt2minmin/chi2minmin;

  chi2_second[0]->Fill(chi2second,weight);
  m_dEqsurchi2[0]->Fill(chi2minmin,dEq,weight);
  m_dElsurchi2[0]->Fill(chi2minmin,dEl,weight);
  m_dmWsurchi2[0]->Fill(chi2minmin,dmW,weight);
  m_dmtsurchi2[0]->Fill(chi2minmin,dmt,weight);
  m_dEdmW[0]->Fill(dEq,dmW,weight);

  if (isEleEvent) {
    chi2_second[1]->Fill(chi2second,weight);
    m_dEqsurchi2[1]->Fill(chi2minmin,dEq,weight);
    m_dElsurchi2[1]->Fill(chi2minmin,dEl,weight);
    m_dmWsurchi2[1]->Fill(chi2minmin,dmW,weight);
    m_dmtsurchi2[1]->Fill(chi2minmin,dmt,weight);
    m_dEdmW[1]->Fill(dEq,dmW,weight);
  } else {
    chi2_second[2]->Fill(chi2second,weight);
    m_dEqsurchi2[2]->Fill(chi2minmin,dEq,weight);
    m_dElsurchi2[2]->Fill(chi2minmin,dEl,weight);
    m_dmWsurchi2[2]->Fill(chi2minmin,dmW,weight);
    m_dmtsurchi2[2]->Fill(chi2minmin,dmt,weight);
    m_dEdmW[2]->Fill(dEq,dmW,weight);
  }


  b_EventNumber_ana = EventNumber;
  b_RunNumber_ana = RunNumber;
  b_chi2minmin = chi2minmin;
  b_chi2second = chi2second;
  b_isEleEvent = isEleEvent;
  b_Th1=ThrustT;
  b_Th2=ThrustTT;
  b_HT=HT/1000.; 
  b_sumptjets=sumptjets/1000.;
  b_sumptjetsmu=sumptjetsmu/1000.;
  b_ET=topMET_sumet/1000.;
  b_MET=topMET_et/1000.; 
  b_phiMET=topMET_phi;
  b_PTsubleadingjet=Ptmax2/1e3;
  b_PTleadingjet=Ptmax/1e3;
  b_MTlepneu=mTW/1000.;
  b_dphiminMETjet=dphiminMETjet;
  b_pTWlep=pTWlep.Mod()/1000.;
  b_mjjj=mjjj/1000.;
  b_dRWhadj=dRWhadj;
  b_pTWhad=pTWhad/1000.;
  b_mlepjet=mlepjet/1000.;
  b_mtophad=m2minmin/1000.;
  b_mtoplep=m3minmin/1000.;
  b_EWhad=vWhadminmin.E()/1000;
  b_EWlep=vWlepminmin.E()/1000;
  b_pTtophad=vthminmin.Pt()/1000;
  b_pTtoplep=vtlminmin.Pt()/1000;
  b_pZtophad=vthminmin.Z()/1000;
  b_pZttbar=vttpairchi2.Z()/1000;

  b_ptjbqq=jbqq.Pt()/1000.;
  b_ptjq1W=jq1W.Pt()/1000.;
  b_ptjq2W=jq2W.Pt()/1000.;
  b_ptjben=jben.Pt()/1000.;
  b_etajbqq=jbqq.Eta();
  b_etajq1W=jq1W.Eta();
  b_etajq2W=jq2W.Eta();
  b_etajben=jben.Eta();
  b_phijbqq=jbqq.Phi();
  b_phijq1W=jq1W.Phi();
  b_phijq2W=jq2W.Phi();
  b_phijben=jben.Phi();

  b_ptLep=vlep.Pt()/1000.;
  b_etaLep=vlep.Eta();
  b_phiLep=vlep.Phi();
  b_ptNeu=ptNeu/1000.;
  b_etaNeu=etaNeu;
  b_etaMin=etas.at(0);  
  b_etaMax=etas.at(5);
  b_dRqqW=dRqq_afterchi2;
  b_dRmin_eq=dRmineq;
  b_ctet_qtlep=costheta_qtlep;
  b_cos_q1CMw_topboost=cosq1CMw_topboost;
  b_cos_q2CMw_topboost=cosq2CMw_topboost;
  b_cos_lepCMw=cos_lepCMw;
  b_cos_bWenCMt=cos_bWenCMt;
  b_cos_lepCMt=cos_lepCMt;
  b_cos_bneuCMt=cos_bneuCMt;
  b_cosWqq=cosWqq;
  b_cosWen=cosWen;
  b_cosqW1=fabs(cosqW1);
  b_cosqW2=cosqW2;
  b_PWqq=PWqq/1000.;
  b_PWen=PWen/1000.;
  b_P_thad=P_thad/1000.;
  b_P_tlep=P_tlep/1000.;
  b_costpair_lab=costpair_lab;
  b_dRminbNeu=dRminbNeu;
  b_dRminbhadWqq=dRminbhadWqq;
  b_dRminblepWqq=dRminblepWqq;
  b_dRbb=dRbb;
  b_dRminNeuWqq=dRminNeuWqq;
  b_cos_tlepCMtpair=cos_tlepCMtpair;
  b_ptWqq=vWqq.Pt()/1000.;
  b_ptWen=vWen.Pt()/1000.;

  tree_ana->Fill();
  return kTRUE;
}

void ana2011::SlaveTerminate()
{
   // The SlaveTerminate() function is called after all entries or objects
   // have been processed. When running with PROOF SlaveTerminate() is called
   // on each slave server.

}

void ana2011::Terminate()
{
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.

  //TFile *file = new TFile("res.root","RECREATE");
  //TTree *tCop = tree_ana->CloneTree();
  file->cd();
  tree_ana->Write();
  
  TIter next(fOutput);
  while(TObject* obj = next() ) {
    TString tobj = obj->IsA()->GetName();
    obj->Write();
  }

  //tree_ana->Write();
  file->Close();

  fclose(EvtListC2);
  fclose(EvtListC3);
  fclose(EvtListC4);
  fclose(EvtListC5);
  fclose(EvtListC11);
  fclose(EvtListC12);
  fclose(EvtListC13);
  EvtListC14e.close();
  EvtListC14m.close();

  cout << "iEvtSel = " << iEvtSel << endl;
  cout << "n6DifPartons = " << n6DifPartons << endl;
  cout << "nHyperNasty = " << nHyperNasty << endl;
  cout << "nHasJetInHole = " << nHasJetInHole << endl;
}

/*
#include "/exp/atlas/huonglan/LHAPDF/lhapdf-5.8.4/myInstall/include/LHAPDF/FortranWrappers.h"
#ifdef FC_DUMMY_MAIN
int FC_DUMMY_MAIN() {return l;}
#endif
*/

//Functions to choose events and particles
void ana2011::GetRunNumberBranch(int entry) 
{
  b_RunNumber->GetEntry(entry);
  b_EventNumber->GetEntry(entry);
}

//void ana2011::GetSelBranch(int entry, UInt_t rnumber,int sel) 
void ana2011::GetSelBranch(int entry, int sel) 
{
  if (sel == 0) {
    if ((!RealData) && (!mcWjets)) {
      b_mcevt_pdf_id1->GetEntry(entry);
      b_mcevt_pdf_id2->GetEntry(entry);
      b_mcevt_pdf_x1->GetEntry(entry);
      b_mcevt_pdf_x2->GetEntry(entry);
      b_mcevt_pdf_scale->GetEntry(entry);
      b_mcevt_pdf1->GetEntry(entry);
      b_mcevt_pdf2->GetEntry(entry);
      b_mcevt_weight->GetEntry(entry);
    }
    //triggers
    b_lbn->GetEntry(entry);
    b_larError->GetEntry(entry);
    b_EF_mu18->GetEntry(entry);
    b_EF_e20_medium->GetEntry(entry);
  }

  if (sel == 1) {
    //vertex
    b_vxp_n->GetEntry(entry);
    b_vxp_z->GetEntry(entry);
    b_vxp_type->GetEntry(entry);
    b_vxp_nTracks->GetEntry(entry);
  }

  if (sel == 2) {
    //JetInHole
    b_jet_antiKtZ4Track_n->GetEntry(entry);
    b_jet_antiKtZ4Track_pt->GetEntry(entry);
    b_jet_antiKtZ4Track_eta->GetEntry(entry);
    b_jet_antiKtZ4Track_phi->GetEntry(entry);
    b_topMET_sumet->GetEntry(entry);
    b_topMET_etx->GetEntry(entry);
    b_topMET_ety->GetEntry(entry); 
    b_topMET_et->GetEntry(entry);
    b_topMET_phi->GetEntry(entry);
    //b_MET_Truth_NonInt_etx->GetEntry(entry);
    //b_MET_Truth_NonInt_ety->GetEntry(entry);
  }
  
  if (sel == 3){
    //electron
    b_el_OQ->GetEntry(entry);
    b_el_cl_E->GetEntry(entry);
    b_el_EF_dr->GetEntry(entry);
    b_el_cl_pt->GetEntry(entry);
    b_el_cl_phi->GetEntry(entry);
    b_el_cl_eta->GetEntry(entry);
    b_el_author->GetEntry(entry);
    b_el_tight->GetEntry(entry);
    b_el_medium->GetEntry(entry);
    b_el_Etcone20->GetEntry(entry);
    b_el_etas2->GetEntry(entry);
    b_el_etap->GetEntry(entry);
    b_el_Etcone20_corrected->GetEntry(entry);
    b_el_Etcone20_pt_corrected->GetEntry(entry);
    b_el_trackpt->GetEntry(entry);
    b_el_tracketa->GetEntry(entry);
    b_el_trackphi->GetEntry(entry);
    b_el_isEM->GetEntry(entry);
    b_el_trackd0->GetEntry(entry);    
    b_el_trackz0->GetEntry(entry);    
    b_el_tracktheta->GetEntry(entry);
    b_el_trackqoverp->GetEntry(entry);

    //muon
    b_mu_me_theta->GetEntry(entry); 
    b_mu_me_qoverp->GetEntry(entry);

    b_mu_id_d0->GetEntry(entry);	  
    b_mu_id_z0->GetEntry(entry);	  
    b_mu_id_phi->GetEntry(entry);	  
    b_mu_id_theta->GetEntry(entry); 
    b_mu_id_qoverp->GetEntry(entry);

    b_mu_E->GetEntry(entry);
    b_mu_EFCB_dr->GetEntry(entry);
    b_mu_pt->GetEntry(entry);
    b_mu_phi->GetEntry(entry);
    b_mu_eta->GetEntry(entry);
    //b_mu_m->GetEntry(entry);
    b_mu_tight->GetEntry(entry);
    b_mu_author->GetEntry(entry);
    b_mu_isCombinedMuon->GetEntry(entry);
    b_mu_expectBLayerHit->GetEntry(entry);
    b_mu_nBLHits->GetEntry(entry);
    b_mu_nPixHits->GetEntry(entry);
    b_mu_nPixelDeadSensors->GetEntry(entry);
    b_mu_nSCTHits->GetEntry(entry);
    b_mu_nSCTDeadSensors->GetEntry(entry);
    b_mu_nPixHoles->GetEntry(entry);
    b_mu_nSCTHoles->GetEntry(entry);
    b_mu_nTRTHits->GetEntry(entry);
    b_mu_nTRTOutliers->GetEntry(entry);
    b_mu_ptcone30->GetEntry(entry);
    b_mu_etcone30->GetEntry(entry);

    //jet : because there is mu-jet overlap removal
    b_jet_E->GetEntry(entry);
    b_jet_n->GetEntry(entry);
    b_jet_pt->GetEntry(entry);
    b_jet_eta->GetEntry(entry);
    b_jet_phi->GetEntry(entry);
    b_jet_m->GetEntry(entry);
  }

  if (sel == 4){
    //cout << "getselbranch 0" << endl;
    b_trig_EF_el_n->GetEntry(entry);
    //cout << "getselbranch 1" << endl;
    b_trig_EF_el_EF_e20_medium->GetEntry(entry);
    //cout << "getselbranch 2" << endl;
    b_trig_EF_el_eta->GetEntry(entry);
    //cout << "getselbranch 3" << endl;
    b_trig_EF_el_phi->GetEntry(entry);
    //cout << "getselbranch 4" << endl;
    b_trig_L2_combmuonfeature_n->GetEntry(entry);
    //cout << "getselbranch 5" << endl;
    b_trig_L2_combmuonfeature_L2_mu18->GetEntry(entry);
    //cout << "getselbranch 6" << endl;
    b_trig_L2_combmuonfeature_eta->GetEntry(entry);
    //cout << "getselbranch 7" << endl;
    b_trig_L2_combmuonfeature_phi->GetEntry(entry);
    //cout << "getselbranch 8" << endl;
    b_trig_EF_trigmuonef_n->GetEntry(entry);
    //cout << "getselbranch 9" << endl;
    b_trig_EF_trigmuonef_track_n->GetEntry(entry);
    //cout << "getselbranch 10" << endl;
    b_trig_EF_trigmuonef_track_CB_eta->GetEntry(entry);
    //cout << "getselbranch 11" << endl;
    b_trig_EF_trigmuonef_track_CB_phi->GetEntry(entry);
    //cout << "getselbranch 12" << endl;
    b_trig_EF_trigmuonef_track_CB_pt->GetEntry(entry);
    //cout << "getselbranch 13" << endl;
  }

  if (sel == 5) {
    //jets
    b_jet_EMJES_EtaCorr->GetEntry(entry);
    b_jet_emscale_eta->GetEntry(entry);
    b_jet_emfrac->GetEntry(entry);
    b_jet_hecf->GetEntry(entry);
    b_jet_LArQuality->GetEntry(entry);
    b_jet_HECQuality->GetEntry(entry);
    b_jet_Timing->GetEntry(entry);
    b_jet_sumPtTrk->GetEntry(entry);
    b_jet_fracSamplingMax->GetEntry(entry);
    b_jet_NegativeE->GetEntry(entry);
    b_jet_flavor_weight_SV0->GetEntry(entry);
    b_jet_flavor_weight_Comb->GetEntry(entry);
    b_jet_flavor_weight_JetFitterCOMBNN->GetEntry(entry);
    if (!RealData)
      b_jet_flavor_truth_label->GetEntry(entry);

    b_jet_isBadLoose->GetEntry(entry);
    b_jet_BCH_CORR_JET->GetEntry(entry);
    b_jet_BCH_CORR_CELL->GetEntry(entry);
  }

  if ((!RealData) && (doMatch) && sel == 6) {
    //MC informations
    b_mc_n->GetEntry(entry);
    b_mc_pt->GetEntry(entry);
    b_mc_m->GetEntry(entry);
    b_mc_eta->GetEntry(entry);
    b_mc_phi->GetEntry(entry);
    b_mc_status->GetEntry(entry);
    b_mc_barcode->GetEntry(entry);
    b_mc_pdgId->GetEntry(entry);
    b_mc_child_index->GetEntry(entry);  
    b_mu_truth_mothertype->GetEntry(entry);
    b_mu_truth_barcode->GetEntry(entry);
    b_el_truth_barcode->GetEntry(entry);
    b_el_truth_mothertype->GetEntry(entry);
  }

}

bool ana2011::AtLeastOneGoodPrimaryVertex(int nvtx)
{
  int igoodvertex = 0;

  if (vxp_n >= 1) {
    if (vxp_type->at(0) == 1 && vxp_nTracks->at(0) > nvtx)
      igoodvertex++;
  }

  if (igoodvertex < 1) return false;
  
  return true;
}

bool ana2011::OneLooseBadJet(double PtminLooseBadJet, double EminLooseBadJet)
{
  int nLooseBadJet = 0;

  for (int i = 0; i < jet_n; i++) {
    
    double emfA      = jet_emfrac->at(i);
    double hecfA     = jet_hecf->at(i);
    double larqA     = jet_LArQuality->at(i);
    double hecqA     = jet_HECQuality->at(i);
    double timeA     = jet_Timing->at(i);
    double sumpttrkA = jet_sumPtTrk->at(i);
    double etaA      = jet_emscale_eta->at(i);
    double ptA       = jet_pt->at(i);
    double fmaxA     = jet_fracSamplingMax->at(i);
    double negEA     = jet_NegativeE->at(i);

    double E         = jet_E->at(i);

    bool loosebadjet = JetID::isBadJet(JetID::LooseBad,emfA,hecfA,larqA,hecqA,timeA,sumpttrkA,etaA,ptA,fmaxA,negEA);
   
    //bool loosebadjet = jet_isBadLoose->at(i) ? true : false;
    if (loosebadjet && ptA>PtminLooseBadJet && E>EminLooseBadJet)
      nLooseBadJet ++;
  }

  if (nLooseBadJet >= 1) return true;

  return false;
}

double ana2011::VetoTrackJetInHole()
{
  double sumptjetHole = 0.;
  bool HasJetInHole = false;
  for (int i = 0; i < jet_antiKtZ4Track_n; i++){
    double pt  = jet_antiKtZ4Track_pt->at(i);
    double eta = jet_antiKtZ4Track_eta->at(i);
    double phi = jet_antiKtZ4Track_phi->at(i);

    if (phi>-0.9 && phi<-0.47 && eta>=-0.2 && eta<=1.6){
      HasJetInHole = true;
      sumptjetHole += pt/1000;
    }
  }
  
  if (HasJetInHole) nHasJetInHole++;
  return sumptjetHole;
}

bool ana2011::GoodElectron(int iel) 
{
  if (el_truth_barcode->at(iel)>30) return false;
  
  m_looseEle->Fill(0);  

  float e_cl   = el_cl_E->at(iel);
  float eta_tr = el_tracketa->at(iel);
  float eta_cl = el_cl_eta->at(iel);
  float phi_cl = el_cl_phi->at(iel);
  float pt     = e_cl/cosh(eta_tr);

  m_ptElSm = pt;

  // Author
  if (el_author->at(iel) != 1 && el_author->at(iel) != 3) return false;
  m_looseEle->Fill(1);

  // Acceptance (remove crack also):
  float aeta   = fabs(el_cl_eta->at(iel));

  if (aeta > 2.47)                return false;
  m_looseEle->Fill(2);

  if (aeta > 1.37 && aeta < 1.52) return false;
  m_looseEle->Fill(3);

  //Energy smearing
  double eEsmear = e_cl;
  if (!RealData && m_smearMC){
    m_eleEscalesmear->SetRandomSeed(EventNumber);
    eEsmear = e_cl*m_eleEscalesmear->getSmearingCorrection(eta_cl,e_cl,0,true,"2011");
  } else if (RealData && m_corrData)
    eEsmear = m_eleEscalesmear->applyEnergyCorrection(eta_cl,phi_cl,e_cl,pt,0,"2011");
    
  float ptS = eEsmear/cosh(eta_tr);
  
  if (ptS < 25e3) return false;
  m_looseEle->Fill(4);
  
  //Check OQ
  int checkOQ = el_OQ->at(iel);
  if ((checkOQ&1446)!=0) return false;
  m_looseEle->Fill(5);

  // ID
  if (el_tight->at(iel) != 1) return false;
  m_looseEle->Fill(6);

  //unsigned int isem = el_isEM->at(iel);
  //egammaPID
  //const unsigned int tight_trackmatch = egammaPID::ElectronTight_WithTrackMatch;
  //if ((tight_trackmatch&isem)!= 0) return false;
  //m_looseEle->Fill(7);
  
  // Isolation
  int nPV=0;
  for(unsigned int i=0; i<vxp_nTracks->size(); i++) {
    if(vxp_nTracks->at(i) >= 2)   nPV++;
  }
  double etcone20_ptcorr = CaloIsoCorrection:: GetPtNPVCorrectedIsolation(nPV,e_cl,el_etas2->at(iel),el_etap->at(iel),el_cl_eta->at(iel),20,(!RealData),el_Etcone20->at(iel),CaloIsoCorrection::ELECTRON);

  if (etcone20_ptcorr > 3.5e3) return false;
  m_looseEle->Fill(8);

  if (!RealData && m_ApplyHoleVeto && hasLArHole){
    if (myOQ.checkOQClusterElectron(180614,el_cl_eta->at(iel),el_cl_phi->at(iel))==3) return false;
  }
  m_looseEle->Fill(9);

  fprintf(testptSEl1,"%10.2f\n",ptS);

  m_ptElSm = ptS;
  
  return true;  
}

bool ana2011::GoodMuon(int imu) 
{
  m_looseMuon->Fill(0);

  // ID
  if (mu_tight->at(imu) != 1)      return false;
  m_looseMuon->Fill(1);

  // Author
  if (mu_author->at(imu) != 12)     return false;
  m_looseMuon->Fill(2);
    
  //acceptance
  double eta = fabs(mu_eta->at(imu));  

  if (eta > 2.5)                    return false;
  m_looseMuon->Fill(3);

  //smear pt
  double pt = mu_pt->at(imu);
  m_ptMuSm = pt;
  double ptSmear = pt;
  if (!RealData && m_smearMC){
    double ptme = sin(mu_me_theta->at(imu))/fabs(mu_me_qoverp->at(imu));
    double ptid = sin(mu_id_theta->at(imu))/fabs(mu_id_qoverp->at(imu));
    m_muPsmear->SetSeed(EventNumber,imu);
    m_muPsmear->Event(ptme,ptid,pt,eta); 
    ptSmear = m_muPsmear->pTCB();
  }
  
  if (ptSmear < 20e3)                return false;
  m_looseMuon->Fill(4);
  
  //if ((mu_expectBLayerHit->at(imu)==1 && mu_nBLHits->at(imu)<1) || mu_expectBLayerHit->at(imu)>1)
  if (!(!mu_expectBLayerHit->at(imu) || mu_nBLHits->at(imu) > 0))
    return false;
  m_looseMuon->Fill(5);

  if ((mu_nPixHits->at(imu) + mu_nPixelDeadSensors->at(imu)) <= 1)
    return false;
  m_looseMuon->Fill(6);
  
  if ((mu_nSCTHits->at(imu) + mu_nSCTDeadSensors->at(imu)) < 6)
    return false;
  m_looseMuon->Fill(7);
  
  if ((mu_nPixHoles->at(imu)+ mu_nSCTHoles->at(imu)) >= 3)
    return false; 
  m_looseMuon->Fill(8);
  
  int n = mu_nTRTHits->at(imu) + mu_nTRTOutliers->at(imu);
  //int n = mu_nTRTHits->at(imu);
  
  if (eta < 1.9){
    if (n <= 5 || float(mu_nTRTOutliers->at(imu))/n >= 0.9)
      return false;
    m_looseMuon->Fill(9);
  }
  else {
    if (n > 5) {
      if (float(mu_nTRTOutliers->at(imu))/n >= 0.9)
	return false; 
      m_looseMuon->Fill(10);
    }
  }

  //Isolation
  if (mu_ptcone30->at(imu) > 4e3)
    return false;
  m_looseMuon->Fill(11);

  if (mu_etcone30->at(imu) > 4e3)
    return false;
  m_looseMuon->Fill(12);

  //overlap removal muons-jets
  bool overlap_mujet = false;
  TLorentzVector v4m;
  v4m.SetPtEtaPhiM(ptSmear,mu_eta->at(imu),mu_phi->at(imu),105.);
  
  for (int i = 0; i < jet_n; i++) {
    if (jet_pt->at(i)>20e3 && jet_E->at(i)>=0){
      TLorentzVector v4j;
      v4j.SetPtEtaPhiM(jet_pt->at(i),jet_eta->at(i),jet_phi->at(i),jet_m->at(i));
      
      double dRmj = v4j.DeltaR(v4m);
      if (dRmj < 0.4) overlap_mujet = true;
    }
  }

  if (overlap_mujet)
    return false;
  m_looseMuon->Fill(13);

  fprintf(testptSMu1,"%10.2f\n",ptSmear);

  m_ptMuSm = ptSmear;

  return true;
}

bool ana2011::GoodJets(int ij) 
{

  // Acceptance (remove crack also):
  float e    = jet_E->at(ij);
  float aeta = fabs(jet_emscale_eta->at(ij)+jet_EMJES_EtaCorr->at(ij));

  float pt   = jet_pt->at(ij);

  if (e < 0)       return false;

  if (aeta >= 2.5)  return false;
  if (pt <= 25e3)    return false;
  
  return true;  
}



  
